using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using System.Xml;
using System.Web.Services.Protocols;

using SCI.Gateway.InterfaceTypes;
using SCI.Gateway.ThirdParty;

namespace SCI.Gateway.ThirdPartyInterface
{
	/// <summary>
	/// Summary description for Service1.
	/// </summary>
	public class ThirdPartyInterface : System.Web.Services.WebService
	{
		private string xmlDataStoreLocation;

		public ThirdPartyInterface()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();

			xmlDataStoreLocation = "../config/LogonInfo.xml";
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		[WebMethod]
		//public bool getLogonPage(string instance, string progName, string progId, string patientKey, string token)
		public bool getLogonPage(string instance, string progName, string progId, string patientKey, string token, out string logonPage)
		{
			XmlQualifiedName errorCode;
			Acknowledge ackObject;
			XmlNode xmlDetailNode;
			ThirdParty.Interface interfaceObject;
			bool success;
			string dataStore;

		//	string logonPage;

			success = false;
			ackObject = new Acknowledge();

			logonPage = "";

			dataStore = Context.Request.MapPath(xmlDataStoreLocation);

			interfaceObject = new ThirdParty.Interface(dataStore);

			ackObject = interfaceObject.getLogonPage(instance, progName, progId, patientKey, token, out logonPage);

			if (ackObject.Code == 0)
			{
				success = true;
			}
			else
			{
				//an error has occurred, throw a soap exception
				xmlDetailNode = buildSoapException(ackObject.Code.ToString(), ackObject.Detail, ackObject.UserMsg);

				errorCode = new XmlQualifiedName("Error");
				
				throw new SoapException(ackObject.UserMsg, errorCode, ackObject.Source, xmlDetailNode);

			}

			return success;
		}

		[WebMethod]
		//public bool getInstances()
		public bool getInstances(out string[] instanceNames)
		{
			XmlNode xmlDetailNode;
			XmlQualifiedName errorCode;
			Acknowledge ackObject;
			ThirdParty.Interface interfaceObject;
			bool success;
			string dataStore;

			dataStore = Context.Request.MapPath(xmlDataStoreLocation);

			interfaceObject = new ThirdParty.Interface(dataStore);

			ackObject = interfaceObject.getInstances(out instanceNames);

			if (ackObject.Code == 0)
			{
				success = true;
			}
			else
			{
				//an error has occurred, throw a soap exception
				xmlDetailNode = buildSoapException(ackObject.Code.ToString(), ackObject.Detail, ackObject.UserMsg);

				errorCode = new XmlQualifiedName("Error");
				
				throw new SoapException(ackObject.UserMsg, errorCode, ackObject.Source, xmlDetailNode);

			}

			return success;			
		}

		[WebMethod]		
		//public bool logon(string username, string password, string instance)
		public bool logon(string username, string password, string instance, out string token, out int pwdStatusCode)
		{
			XmlNode xmlDetailNode;
			XmlQualifiedName errorCode;
			Acknowledge ackObject;
			ThirdParty.Interface interfaceObject;
			bool success;
			string dataStore;
//			string token;
//			int pwdStatusCode;

			token = "";
			pwdStatusCode = 0;

			dataStore = Context.Request.MapPath(xmlDataStoreLocation);

			interfaceObject = new ThirdParty.Interface(dataStore);

			ackObject = interfaceObject.logon(username, password, instance, out token, out pwdStatusCode);

			success = (pwdStatusCode >= 0);

			if (ackObject.Code != 0)
			{
				//an error has occurred, throw a soap exception
				xmlDetailNode = buildSoapException(ackObject.Code.ToString(), ackObject.Detail, ackObject.UserMsg);

				errorCode = new XmlQualifiedName("Error");
				
				throw new SoapException(ackObject.UserMsg, errorCode, ackObject.Source, xmlDetailNode);

			}

			return success;			
		}

		[WebMethod]
		public bool changePassword(string username, string oldpassword, string newpassword, string instance)
		{
			XmlNode xmlDetailNode;
			XmlQualifiedName errorCode;
			Acknowledge ackObject;
			ThirdParty.Interface interfaceObject;
			bool success;
			string dataStore;

			dataStore = Context.Request.MapPath(xmlDataStoreLocation);

			interfaceObject = new ThirdParty.Interface(dataStore);

			ackObject = interfaceObject.changePassword(username, oldpassword, newpassword, instance);

			if (ackObject.Code == 0)
			{
				success = true;
			}
			else
			{
				//an error has occurred, throw a soap exception
				xmlDetailNode = buildSoapException(ackObject.Code.ToString(), ackObject.Detail, ackObject.UserMsg);

				errorCode = new XmlQualifiedName("Error");
				
				throw new SoapException(ackObject.UserMsg, errorCode, ackObject.Source, xmlDetailNode);

			}

			return success;			

		}	

		public XmlNode buildSoapException(string Code, string Details, string UserMsg)
		{
			System.Xml.XmlDocument doc = new System.Xml.XmlDocument();

			System.Xml.XmlNode node = doc.CreateNode(XmlNodeType.Element, SoapException.DetailElementName.Name, SoapException.DetailElementName.Namespace);


			// Build specific details for the SoapException.
			// Add first child of detail XML element.
			System.Xml.XmlNode details = doc.CreateNode(XmlNodeType.Element, "ErrorDetails", "http://tempuri.org/");
			System.Xml.XmlNode detailsChild1 = doc.CreateNode(XmlNodeType.Element, "Code", "http://tempuri.org/");
			System.Xml.XmlNode detailsChild2 = doc.CreateNode(XmlNodeType.Element, "Detail", "http://tempuri.org/");
			System.Xml.XmlNode detailsChild3 = doc.CreateNode(XmlNodeType.Element, "UserMsg", "http://tempuri.org/");

			details.AppendChild(detailsChild1);
			details.AppendChild(detailsChild2);
			details.AppendChild(detailsChild3);

			detailsChild1.InnerText = Code;
			detailsChild2.InnerText = Details;
			detailsChild3.InnerText = UserMsg;

			// Append the two child elements to the detail node.
			node.AppendChild(details);

			return node;
		}
	}
}
