<%   
'============================================================================================
' File name:    common.asp
' Description:  This file provides functions and session constants commonly used by ASP files.
'============================================================================================
    
'=============================================
' Page Variables
'=============================================

'Variables to configure the header.asp file
Dim g_strPageTitle      'As String
Dim g_strStyleSheet     'As String
Dim g_strTargetFrame    'As String
Dim g_strMetaPairs      'As String  (comma separated meta-pairs)
Dim g_strVBScripts      'As String  (comma separated file list)
Dim g_strJavaScripts    'As String  (comma separated file list)
Dim g_blnCloseHead      'As Boolean (whether </head> tag is inserted by headhtml.asp)

'Initialise page variables
g_strTargetFrame = "_top"
g_strMetaPairs   = ""
g_strVBScripts   = ""
g_strJavaScripts = ""
g_blnCloseHead   = True

'Clear the error object - any reportable / trapped should be stored in appropriate Session variables
Err.Clear

Function GetFormValue(strItem, strValue) 'As Boolean
'
' Description:  Wrapper function for looking for a particular Request.Form item.
' Parameters:   strItem  - the form item to look for
'               strValue - the parameter value in the form
' Returns:      True if the parameter is set, False otherwise
'

    strValue = ""
    If Not Request.Form Is Nothing Then
        strValue = CStr(Request.Form(strItem))
    End If
    GetFormValue = Len(strValue) > 0 
    
End Function

Function GetFormValueDefault(strItem, strValue, strType, varDefault) 'As Boolean
'
' Description:  Wrapper function for looking for a particular Request.Form item.
' Parameters:   strItem - the form item to look for
'               strValue - the parameter value in the form
'               strType - the VB type to convert to
'               varDefault - the default value to return
' Returns:      True if a value is obtained, False if the default is used
'
    
    If GetFormValue(strItem, strValue) Then

        'Convert it to the correct type
        Select Case LCase(strType)
			Case "int", "integer"   :    strValue = CInt(strValue)
            Case "long", "lng"      :    strValue = CLng(strValue)
            Case "date"             :    strValue = CDate(strValue)
            Case "double", "dbl"    :    strValue = CDbl(strValue)
            Case "single", "sng"    :    strValue = CSng(strValue)
            Case "boolean", "bool"  :    strValue = CBool(strValue)
        End Select
        GetFormValueDefault = True
        
    Else 'apply the default
        strValue = varDefault
        GetFormValueDefault = False
    End If
    
End Function

Function MonthName(intMonth, blnShort) 'As String
'
' Description:  Output a month name given a number - replaces VB script version.
' Parameters:   intMonth - the month number (from 1 to 12) 
'               blnShort - if true, return a 3 character abbreviation
' Returns:      The month name
'

    Dim strMonth        'As String
    
    If blnShort Then
        Select Case intMonth
            Case  1: strMonth = "Jan"
            Case  2: strMonth = "Feb"
            Case  3: strMonth = "Mar"
            Case  4: strMonth = "Apr"
            Case  5: strMonth = "May"
            Case  6: strMonth = "Jun"
            Case  7: strMonth = "Jul"
            Case  8: strMonth = "Aug"
            Case  9: strMonth = "Sep"
            Case 10: strMonth = "Oct"
            Case 11: strMonth = "Nov"
            Case 12: strMonth = "Dec"
            Case Else: strMonth = "Huh"
        End Select
    Else
        Select Case intMonth
            Case  1: strMonth = "January"
            Case  2: strMonth = "February"
            Case  3: strMonth = "March"
            Case  4: strMonth = "April"
            Case  5: strMonth = "May"
            Case  6: strMonth = "June"
            Case  7: strMonth = "July"
            Case  8: strMonth = "August"
            Case  9: strMonth = "September"
            Case 10: strMonth = "October"
            Case 11: strMonth = "November"
            Case 12: strMonth = "December"
            Case Else: strMonth = "Huh??"
        End Select
    End If
    
    MonthName = strMonth

End Function

Sub TransformXMLStylesheet(objXML, strStylesheetPath, strTransformResult)
'
' Description:  Transforms the XML using the stylesheet at strStylesheetURL to produce HTML.
' Parameters:	objXML             - the XML to be transformed
'               strStylesheetPath  - the local path of the XSL/XSLT to use
' Returns:      strTransformResult - the resultant output (usually HTML)
'
    Dim strURL          'As String
    Dim strTransform    'As String
    Dim objXSL          'As MSXML2.DOMDocument40

    'Initialise output
    strTransformResult = ""

    'Check for no stylesheet provided    
    If Len(CStr(strStylesheetPath)) = 0 Then
        With Response
            .Write("A problem occurred loading the stylesheet")
            .Write(" TransformXMLStylesheet() was provided a blank stylesheet path")
            .Write(" -1")
            .End
        End With
        Exit Sub
    End If

    'Wrap error handling
    On Error Resume Next

        Err.Clear

        'Load the stylesheet
        Set objXSL = Server.CreateObject("MSXML2.DOMDocument.4.0")

        If Err.Number > 0 Then
            With Response
                .Write("A problem occurred creating the stylesheet object")
                .Write(" Server.CreateObject(MSXML2.DOMDocument.4.0) failed: " & Err.Description)
                .Write(" " & Err.Number)
                .End
            End With         

            Set objXSL = Nothing
        End If

        If Not objXSL Is Nothing Then
            objXSL.Async = False
            objXSL.ValidateOnParse = False
            objXSL.Load(strStylesheetPath)

            'Check for a scripting error
            If Err.Number > 0 Then
                With Response
                    .Write("A problem occurred loading the stylesheet")
                    .Write(" objXSL.Load(" & CStr(strStylesheetPath) & ") failed: " & Err.Description)
                    .Write(" " & Err.Number)
                    .End
                End With

            'Check for a DOM parsing error
            ElseIf objXSL.ParseError.ErrorCode <> 0 Then
                With Response
                    .Write("A problem occurred loading the stylesheet")
                    .Write(" objXSL.Load(" & CStr(strStylesheetPath) & ") parsing error: " & objXSL.ParseError.Reason)
                    .Write(" " & objXSL.ParseError.ErrorCode)
                    .End
                End With
            'Transform objXML using the stylesheet
            Else
                ' Set the namespace, this is required for MSXML 4.0
                Call objXSL.setProperty("SelectionLanguage", "XPath") 
                Call objXSL.setProperty("SelectionNamespaces", "xmlns:xsl='http://www.w3.org/1999/XSL/Transform'")             
                If Err.Number > 0 Then
                    With Response
                        .Write("A problem occurred setting the stylesheet namespaces")
                        .Write(" objXSL.setProperty() failed: " & Err.Description)
                        .Write(" " & Err.Number)
                        .End
                    End With
                End If
                
                If Not SetSpellCheckToStylesheet(objXSL, ENABLE_SPELLCHECK) Then
                    ' Stylesheets like BuildPageTitle.xslt will not have the spell check attribute so just ignore if fails.
                End If

                strTransformResult = objXML.TransformNode(objXSL)

                'Check for a scripting error
                If Err.Number > 0 Then
                    With Response
                        .Write("A problem occurred applying the stylesheet")
                        .Write(" objXML.TransformNode(" & CStr(strStylesheetPath) & ") failed: " & Err.Description)
                        .Write(" " & Err.Number)
                        .End
                    End With

                'Check for empty output
                ElseIf Len(strTransformResult) = 0 Then
                    With Response
                        .Write("Applying the stylesheet resulted in no output information")
                        .Write(" objXML.TransformNode(" & CStr(strStylesheetPath) & ") returned no data")
                        .Write(" -1")
                        .End
                    End With
                End If
            End If
                    
        End If
    
    On Error Goto 0
        
    'Cleanup
    Set objXSL = Nothing
    
End Sub

Function SetSpellCheckToStylesheet(objXSL, enableSpellCheck)
'
' Description:  Sets the enablespellcheck variable within the protocol stylesheet.
' Parameters:	objXSL - the stylesheet
'               enableSpellCheck  - the value to use
' Returns:      True if set successfully, false otherwise
'
    Dim objVar
    Dim blnSuccess 
    	
    blnSuccess = False
    If Not objXSL Is Nothing Then
        On Error Resume Next
        Set objVar = objXSL.selectSingleNode("/xsl:stylesheet/xsl:variable[@name='enablespellcheck']")
        If Not objVar Is Nothing Then
            Call objVar.setAttribute("select", "'" & LCase(CStr(ENABLE_SPELLCHECK)) & "'")
            blnSuccess = True
        End If
    End If

    SetSpellCheckToStylesheet = blnSuccess
End Function

Sub BuildPageTitle(objMsgXML, buildPageTitleXsltPath, strTitle)
'
' Description:	Will set the view letter page title based on the content (and type) of the message being viewed.  Generates
'				title using BuildPageTitle.xslt defined on a per message basis.  No default is used, e.g. if no xslt file found, as
'				the information will vary depending of the message type.
'
' Parameters:	objMsgXML               - the DOM of the message being viewed
'				buildPageTitleXsltPath  - the message type for this message
'				strTitle                - (out) used to return the apt value for the title
'
    If(FileExists(buildPageTitleXsltPath)) Then
        Call TransformXMLStylesheet(objMsgXML, buildPageTitleXsltPath, strTitle)
    End If
       
End Sub                

Function FileExists(filePath)         'As Boolean
'
' Description:	Checks if a file exists
'
' Parameters:	filePath - the path and filename to check
'
' Returns:		Returns true if the path specified corresponds to an existing file, otherwise false
'
    Dim fileSystemObj
    Set fileSystemObj = CreateObject("Scripting.FileSystemObject")
    FileExists = fileSystemObj.fileexists(filePath)
End Function
%>