Option Explicit
'============================================================================================
' File name:    gpass.vbs
' Description:  Contains functionality for integration with Gpass.
'============================================================================================

Const MAX_RECORDS           = 500       'default max number of records returned for dynaTables
Const COLUMN_DELIMITER      = "___"     'delimiter used between question names & column specifiers in a form item
Const PROGRESS_BAR_WIDTH    = 200

' Whether to display message boxes with timings for pre-population
Const DISPLAY_TIMES = False
Const DISPLAY_TOTAL_TIME = False

Const EVENTTYPE_READCODE    = 1
Const EVENTTYPE_SCREENING   = 2
Const EVENTTYPE_DRUG        = 3

Const FILTER_ATTRIB_PROPERTY = 0
Const FILTER_ATTRIB_COMPARISON = 1
Const FILTER_ATTRIB_VALUE = 2

Dim m_objPrePopWin, m_objPrePopMsg
Dim m_objClinEventProgMain, m_objClinEventProgDetail, m_objClinEventProgNumbers
Dim m_strPrePopWindowName

Sub PopulateGpassReferral(strProtocolXML, strPatientKey, objInfoForm)

    Dim objXML                      'As MSXML2.DOMDocument40
    Dim objPatient                  'As Patient
    Dim objEncounterReadCodes()     'As Array of ProtocolObject
    Dim objEncounterDrugs()         'As Array of ProtocolObject
    Dim objEncounterScreenings()    'As Array of ProtocolObject
    Dim colClinicalEvents           'As Gpass API exposed 'collection' of clinical events
    Dim objCareReadCodeDict         'As Dictionary object
    Dim arrReadCodeEvents           'As Array of ProtocolObject
    Dim arrScreeningEvents          'As Array of ProtocolObject
    Dim arrDrugs                    'As Array of ProtocolObject
    Dim intNoCurrEncRCEs
    Dim intNoCurrEncSEs
    Dim intNoCurrEncPDs
    Dim strPatientName

    intNoCurrEncRCEs = 0
    intNoCurrEncSEs = 0
    intNoCurrEncPDs = 0

    Call SetPrePopMsg("Loading the protocol XML")

    'Load the XML object
    Set objXML = LoadXMLFromString(strProtocolXML)
    If objXML Is Nothing Then
        Exit Sub
    End If

    Call SetPrePopMsg("Retrieving the patient information")

    'Get objects (only the Patient at this stage) required by both basic & complex mappings (ie, filtered collections)
    Set objPatient = New Patient
    Call objPatient.LetProperty("Key", strPatientKey)

    If objPatient Is Nothing Then
	    Call MsgBox("An error has occurred retrieving the patient with patient key '" & strPatientKey & "'.", , "SCI Protocol Verification Tool")
	    Exit Sub
    Else
	    strPatientName = "<b>" & objPatient.GetProperty("forename") & " " & objPatient.GetProperty("surname") & "</b>"
    End If

    Call SetPrePopMsg("Retrieving clinical events for: " & strPatientName)
    'Get clinical events and adjust for care sessions
    Set colClinicalEvents = objPatient.GetProperty("ClinicalEvents")

    'Test Track 50123 - need to get clinical events in care sessions separately
    'in case they are not returned by the call to the 
    'ClinicalEvents method of the patient object (they may or may not be returned - 
    'we are dealing with a bug in the Gpass API here.  doro
    Set objCareReadCodeDict = GetCareSessReadCodes(colClinicalEvents)

    Call SetPrePopMsg("Populating non-clinical event information for: " & strPatientName)	

    'Populate questions that have basic mappings to a single object
    Call PopulateObjects(objXML.documentElement.selectNodes( _
                            "//question[@objectOld]" & _
                            "          [@objectOld!='ReadCodeEvent' " & _
                            "       and @objectOld!='ScreeningEvent' " & _
                            "       and @objectOld!='PrescribedDrug']"), _
                            objInfoForm, _
                            objPatient)

    Call SetPrePopMsg("Retrieving current encounter clinical events for: " & strPatientName)
    
    'Get the current encounter's clinical events
    'Call GetEncounterEvents(objInfoForm, objEncounterReadCodes, objEncounterDrugs, objEncounterScreenings)

    ' Insert all clinical events, including those in care sessions, into the 3 arrays
    BuildClinicalEventArrays colClinicalEvents, arrReadCodeEvents, arrScreeningEvents, arrDrugs, intNoCurrEncRCEs, _
	    intNoCurrEncSEs, intNoCurrEncPDs
    Call SetPrePopMsg("Populating clinical events for: " & strPatientName)
    Call PopulateClinicalEvents(objXML.documentElement.selectNodes("//question[@objectOld='ReadCodeEvent' or @objectOld='ScreeningEvent' or @objectOld='PrescribedDrug'][filters[@type='GPASS']/filter]"), _
                                    arrReadCodeEvents, arrScreeningEvents, arrDrugs)
    Call SetPrePopMsg("Finished pre-population of information for: " & strPatientName)
    Set objPatient = Nothing
    Set objXML = Nothing

    If WindowStillOpen() Then
        ClosePrePopMsgWindow()
    End If
    
End Sub

Function GetCareSessReadCodes(colClinicalEvents)    'As Dictionary object

    Dim colCareSessions
    Dim objClinicalEvent
    Dim objCareValue
    Dim objReadCodeEvent
    Dim objCareReadCodesDict
    Dim intDictKey
    Dim objCareEvent
    Dim strCareEventKey
    Dim strClinicalEventKey
    Dim objCollectionValue
    Dim colCareEvents

    intDictKey = 1

    Set objCareReadCodesDict = CreateObject("Scripting.Dictionary")

    'Due to a bug in the Gpass API, we have to work from the assumption 
    'that clinical events contained in care sessions
    'may or may not be contained in the clinical events collection 
    'returned by the call to the ClinicalEvents method of the patient object
    'Test Track 50123

    For Each objClinicalEvent In colClinicalEvents
        If TypeName(objClinicalEvent) = "CareSession" Then
            For Each objCareValue in objClinicalEvent.Values
                If TypeName(objCareValue.Value) = "ReadCodeEvent" _
                Or TypeName(objCareValue.Value) = "ScreeningEvent" _
                Or TypeName(objCareValue.Value) = "PrescribedDrug" Then
                    Call objCareReadCodesDict.Add(intDictKey, objCareValue.Value)
                    intDictKey = intDictKey + 1
                Else
                    If TypeName(objCareValue.Value) = "ListCollection" Then
                        set colCareEvents = objCareValue.Value
                        set objCollectionValue = colCareEvents.Item(1)

                        If TypeName(objCollectionValue) = "ReadCodeEvent" _
                        Or TypeName(objCollectionValue) = "ScreeningEvent" _
                        Or TypeName(objCollectionValue) = "PrescribedDrug" Then
                            For Each objCollectionValue in colCareEvents
                                Call objCareReadCodesDict.Add(intDictKey, objCollectionValue)
                                intDictKey = intDictKey + 1
                            Next
                        End If
                    End If
                End If
            Next
        End If
    Next

    intDictKey = 0
    For Each intDictKey in objCareReadCodesDict
        set objCareEvent = objCareReadCodesDict.item(intDictKey)
        strCareEventKey = objCareEvent.Key

        For Each objClinicalEvent in colClinicalEvents
            strClinicalEventKey = objClinicalEvent.Key
            If strClinicalEventKey = strCareEventKey Then
                'The clinical event key is already in the clinical event collection,
                'therefore take it out of the Care Session events dictionary object			
                objCareReadCodesDict.remove(intDictKey)
            End If
        Next
    Next

    Set GetCareSessReadCodes = objCareReadCodesDict

End Function

Sub GetEncounterEvents(objForm, objEncounterReadCodes, objEncounterDrugs, objEncounterScreenings)

    Dim strProperty         'As String
    Dim strEvent            'As String
    Dim intIdx              'As Integer
    Dim intCode             'As Integer
    Dim intDrug             'As Integer
    Dim intScrn             'As Integer
    Dim objProtocolObject   'As ProtocolObject
    Dim objDict             'As Scripting.Dictionary
    Dim objElement          'As DHTML.Element
    Dim arrID               'As Array of String

    intCode = -1
    intDrug = -1
    intScrn = -1
    ReDim objEncounterReadCodes(0)
    ReDim objEncounterDrugs(0)
    ReDim objEncounterScreenings(0)    
    Set objEncounterReadCodes(0) = Nothing
    Set objEncounterDrugs(0) = Nothing
    Set objEncounterScreenings(0) = Nothing

    Set objDict = CreateObject("Scripting.Dictionary")

    For Each objElement In objForm.Elements
        strEvent = ""
        arrID = Split(objElement.id, "_")
        Select Case UBound(arrID)
            Case 1 : strEvent = arrID(0)   : strProperty = arrID(1)    : intIdx = 0
            Case 2 : strEvent = arrID(0)   : strProperty = arrID(1)    : intIdx = CInt(arrID(2))
        End Select

        Select Case strEvent
            Case "ReadCodeEvent"

                If intIdx > intCode Then                
                    ReDim Preserve objEncounterReadCodes(intIdx)
                    Set objDict = CreateObject("Scripting.Dictionary")
                    Set objProtocolObject = New ProtocolObject
                    Call objProtocolObject.Use(objDict)
                    Set objEncounterReadCodes(intIdx) = objProtocolObject
                    intCode = intIdx
                Else 
                    Set objProtocolObject = objEncounterReadCodes(intIdx)
                End If
                Call objProtocolObject.LetProperty(strProperty, objElement.value)

            Case "PrescribedDrug"
                If intIdx > intDrug Then                
                    ReDim Preserve objEncounterDrugs(intIdx)
                    Set objDict = CreateObject("Scripting.Dictionary")
                    Set objProtocolObject = New ProtocolObject
                    Call objProtocolObject.Use(objDict)
                    Set objEncounterDrugs(intIdx) = objProtocolObject
                    intDrug = intIdx
                Else 
                    Set objProtocolObject = objEncounterDrugs(intIdx)
                End If
                Call objProtocolObject.LetProperty(strProperty, objElement.value)

            Case "ScreeningEvent"
                If intIdx > intScrn Then                
                    ReDim Preserve objEncounterScreenings(intIdx)
                    Set objDict = CreateObject("Scripting.Dictionary")
                    Set objProtocolObject = New ProtocolObject
                    Call objProtocolObject.Use(objDict)
                    Set objEncounterScreenings(intIdx) = objProtocolObject
                    intScrn = intIdx
                Else 
                    Set objProtocolObject = objEncounterScreenings(intIdx)
                End If
                Call objProtocolObject.LetProperty(strProperty, objElement.value)                                
        End Select
    Next

    Set objProtocolObject = Nothing
    Set objDict = Nothing

End Sub

Sub PopulateObjects(objQuestionList, objInfoForm, objPatient)

    Dim objApplication          'As ProtocolObject
    Dim objReferral             'As Referral
    Dim objReferringClinician   'As Clinician
    Dim objRegisteredClinician  'As Clinician
    Dim objPremises             'As Premises
    Dim objMatch                'As MSXML2.DOMNode
    Dim objQuestion             'As MSXML2.DOMNode
    Dim objAttribute            'As MSXML2.DOMNamedNodeMap 
    Dim objFormElement          'As DHTML.Element
    Dim objOption               'As DHTML.Element
    Dim strObject               'As String
    Dim strProperty             'As String
    Dim strID                   'As String    
    Dim strValue                'As String
    Dim strType                 'As String
    Dim i                       'As Integer

    'If there are no questions, we don't need to do anything
    If objQuestionList Is Nothing Then
        Exit Sub
    End If
    
    'Get the Application object
    Set objApplication = New ProtocolObject
    Call objApplication.Use(g_objGpassApp)
    
    'Get the Referral object
    Call GetReferralFromInfoForm(objReferral, objInfoForm)

    'Get the RegisteredClinician object
    Set objRegisteredClinician = New Clinician
    Call objRegisteredClinician.Use(objPatient.GetProperty("RegisteredClinician"))

    'Get the ReferringClinician object
    Set objReferringClinician = New Clinician
    If objReferral.GetProperty("AuthorisingClinician") Is Nothing Then
        Call objReferringClinician.Use(objPatient.GetProperty("SeenByClinician"))
    Else
        Call objReferringClinician.Use(objReferral.GetProperty("AuthorisingClinician"))
    End If

    'Get the Premises object
    Set objPremises = New Premises
    
    'Loop through each question
    For Each objQuestion In objQuestionList

        strID = objQuestion.attributes.getNamedItem("id").value

        strType = objQuestion.attributes.getNamedItem("type").value
        On Error Resume Next 'Needed to handle those questions that don't have the objectOld or propertyOld attributes
        strObject = objQuestion.attributes.getNamedItem("objectOld").value
        Set objAttribute = objQuestion.attributes.getNamedItem("propertyOld")
        On Error Goto 0

        If Not objAttribute Is Nothing Then
            strProperty = objAttribute.value
            strValue = Eval("obj" & strObject & ".GetProperty(""" & strProperty & """)")
            Set objAttribute = objQuestion.attributes.getNamedItem("source")
            Set objFormElement = document.forms(FORM_ID).elements.item(strID)

            Select Case strType

                'Tables and dynaTable population is not yet supported for non collection objects
                Case "dynaTable", "table"
                    'do nothing

                'Drop-down combo boxes
                Case "select"
                    Set objMatch = objQuestion.selectSingleNode("option[@source='" & strValue & "' or (not(@source) and @value='" & strValue & "')]")
                    If Not objMatch Is Nothing Then
                        Call SetHTMLSelectValue(objFormElement, objMatch.attributes.getNamedItem("value").value)
                    End If 

                'Radio buttons
                Case "radio"
                    Set objMatch = objQuestion.selectSingleNode("option[@source='" & strValue & "' or (not(@source) and @value='" & strValue & "')]")
                    If Not objMatch Is Nothing Then
                        Call SetHTMLRadioValue(objFormElement, objMatch.attributes.getNamedItem("value").value)
                    End If

                'Checkboxes
                Case "check"
                    If Len(strValue) > 0 Then
                        If LCase(strValue) <> "false" Then
                            If InStr(document.forms(FORM_ID).all(strID).outerHTML, "checked=true") = 0 Then
                                'document.forms(FORM_ID).all(strID).checked = True
                                document.forms(FORM_ID).all(strID).outerHTML = Replace(document.forms(FORM_ID).all(strID).outerHTML, ">", " checked=true>")
                            End If
                        End If
                    End If

                'Text boxes, etc.
                Case Else
                    objFormElement.value = strValue
                    
            End Select            
        End If

        Set objMatch = Nothing
        Set objFormElement = Nothing
        Set objAttribute = Nothing
    Next

    Set objReferral = Nothing
    Set objReferringClinician = Nothing
    Set objRegisteredClinician = Nothing

End Sub

Sub SetHTMLSelectValue(objSelect, strValue)
    Dim objOption
    For Each objOption In objSelect.options
        objOption.selected = (objOption.value = strValue)
    Next
End Sub

Sub SetHTMLRadioValue(objRadio, strValue)
    Dim objOption

    'For some reason using the Checked property seems to work at the time, 
    'but gets reset when control is returned back to the client. So change the 
    'HTML directly instead.
    For Each objOption In objRadio
        If objOption.value = strValue Then
            objOption.checked = true
        End If
    Next
End Sub

Sub PopulateTableRow(objQuestion, strID, intCount, objMatchedEvent)
    Dim objColumn   'As MSXML2.DOMNode
    Dim objMatch
    Dim strValue

    For Each objColumn In objQuestion.selectNodes("column[@propertyOld]")
        If objColumn.getAttribute("type") & "" = "select" Then
            strValue = objMatchedEvent.GetProperty(objColumn.getAttribute("propertyOld") & "")
            Set objMatch = objColumn.selectSingleNode("option[@source='" & strValue & "' or (not(@source) and @value='" & strValue & "')]")
            If Not objMatch Is Nothing Then
                Call SetHTMLSelectValue(document.forms(FORM_ID).all(strID & COLUMN_DELIMITER & objColumn.getAttribute("id") & "" & "_" & CStr(intCount)), _
                                objMatch.getAttribute("value") & "")
            End If
        Else            
            document.forms(FORM_ID).all(strID & COLUMN_DELIMITER & objColumn.getAttribute("id")& "" & "_" & CStr(intCount)).value = objMatchedEvent.GetProperty(objColumn.getAttribute("propertyOld") & "")
        End If
    Next        
End Sub

Sub PopulateClinicalEvents(objQuestionList, arrRCE, arrSE, arrDrug)

    Dim objQuestion             'As MSXML2.DOMNode
    Dim objSubitemList          'As MSXML2.DOMNodeList
    Dim objSubitem              'As MSXML2.DOMNode
    Dim strSubItemType
    Dim strObject
    Dim strID
    Dim strType
    Dim arrClinEvent
    Dim blnMatchesFilter
    Dim intEventType
    Dim intCount                'As Integer
    Dim intQuestion             'As Integer
    Dim intNumberOfQuestions

    If Not objQuestionList Is Nothing Then
        intQuestion = 0
        intNumberOfQuestions = objQuestionList.Length

        Call InitialiseProgressBar(intNumberOfQuestions)

        For Each objQuestion In objQuestionList
            intQuestion = intQuestion + 1

            strID = objQuestion.getAttribute("id") & ""
            strType = objQuestion.getAttribute("type") & ""
            strObject = objQuestion.getAttribute("objectOld") & ""

            Select Case LCase(strObject)
                Case "readcodeevent"
                    intEventType = EVENTTYPE_READCODE
                Case "screeningevent"
                    intEventType = EVENTTYPE_SCREENING
                Case "prescribeddrug"
                    intEventType = EVENTTYPE_DRUG
            End Select                

            Select Case strType
                Case "radio", "select", "table"
                    If strType = "table" Then
                        strSubItemType = "row[filter]"
                        intCount = 1
                    Else
                        strSubItemType = "option"
                    End If

                    Set objSubitemList = objQuestion.selectNodes(strSubItemType)
                    If Not objSubitemList Is Nothing Then
                        For Each objSubitem In objSubitemList
                            If MatchAndPopulate(objQuestion, arrRCE, arrSE, arrDrug, strID, strType, intCount, intEventType, objSubItem, True) Then
                                Exit For
                            End If
                        Next
                    End If

                Case "text", "textArea"
	                MatchAndPopulate objQuestion, arrRCE, arrSE, arrDrug, strID, strType, intCount, intEventType, objQuestion, True
                Case "dynaTable"
	                MatchAndPopulate objQuestion, arrRCE, arrSE, arrDrug, strID, strType, intCount, intEventType, objQuestion, False
            End Select
            
            Call UpdateProgressBar(intQuestion, intNumberOfQuestions)
        Next
        
        Call HideProgressBar()

        Set objQuestion = Nothing
    End If
End Sub

Function FilterMatch(arrClinEvents, objFilterList, intCount, arrMatchedEvents, blnMatchOneEventOnly)
'
' Description:  Checks an array of clinical events to see if any match a list of supplied filters.
' Parameters:   arrClinEvents - the array of events
'               objFilterList - DOMNodeList of filters
'               intCount - Returns count of filters found
'               arrMatchedEvents - Returns events that match (Nothing if none)
'               blnMatchOneEventOnly - True if matching one event, False otherwise
' Returns:      True if match found, False otherwise               
'

    Dim blnMatch            'As Boolean
    Dim strProperty         'As String
    Dim strComparison       'As String
    Dim strFilterValue      'As String
    Dim strActualValue      'As String
    Dim objFilter           'As MSXML2.DOMNode
    Dim objClinEvent
    Dim intMatchedEvents
    Dim objRegExp           'As RegExp
    Dim arrFilterList
    Dim intFilterIndex
    Dim blnFiltersExist

    'By default, don't match to questions that don't have any filters.        
    blnMatch = False

    ' Set initial size of array to hold matched events
    If blnMatchOneEventOnly Then
        ReDim arrMatchedEvents(0)   ' Only one event
    Else
        ReDim arrMatchedEvents(19)  ' Up to 20 events - for dynaTables
    End If

    intMatchedEvents = 0

    If Not objFilterList Is Nothing Then
        'Initialise an event to match questions by default, if it is going to be filtered
        blnMatch = (objFilterList.Length > 0)

        'Build filter array
        arrFilterList = buildFilterArray(objFilterList)		
        blnFiltersExist = (UBound(arrFilterList) > LBound(arrFilterList))

        If (UBound(arrClinEvents) >= LBound(arrClinEvents)) Then
            For Each objClinEvent in arrClinEvents
                If blnFiltersExist Then
                    For intFilterIndex=LBound(arrFilterList) To UBound(arrFilterList)
                        strProperty = arrFilterList(intFilterIndex, FILTER_ATTRIB_PROPERTY)				
                        strFilterValue = arrFilterList(intFilterIndex, FILTER_ATTRIB_VALUE)

                        Select Case strProperty
                            'Handle special filter properties
                            Case "[count]"
                                intCount = CInt(strFilterValue)            
                            Case Else
                                strActualValue = objClinEvent.GetProperty(strProperty)
                                strComparison = arrFilterList(intFilterIndex, FILTER_ATTRIB_COMPARISON)
                                
                                Select Case LCase(strComparison)
                                    'Handle numeric comparisons
                                    Case "equals"
                                        blnMatch = CDbl(strActualValue) =  CDbl(strFilterValue)
                                    Case "notequals"
                                        blnMatch = CDbl(strActualValue) <>  CDbl(strFilterValue)
                                    Case "lessthan"
                                        blnMatch = CDbl(strActualValue) <  CDbl(strFilterValue)
                                    Case "atleast"
                                        blnMatch = CDbl(strActualValue) >= CDbl(strFilterValue)
                                    Case "morethan"
                                        blnMatch = CDbl(strActualValue) >  CDbl(strFilterValue)
                                    Case "atmost"
                                        blnMatch = CDbl(strActualValue) <= CDbl(strFilterValue)

                                    'Handle date comparisons
                                    Case "days"
                                        If strActualValue <> "" Then
                                            blnMatch = DateDiff("d", strActualValue, Date()) <= CInt(strFilterValue)
                                        End If

                                    'Handle string comparisons
                                    Case "pattern", "notpattern"
                                        If IsEmpty(objRegExp) Then
                                            'Initialise the regular expression object
                                            Set objRegExp = New RegExp
                                            objRegExp.Global = False        'only return the first match
                                            objRegExp.IgnoreCase = False    'case-sensitive search
                                        End If		                
                                        objRegExp.Pattern = strFilterValue          'set pattern
                                        blnMatch = objRegExp.Test(strActualValue)   'execute the search test

                                        If LCase(strComparison) = "notpattern" Then blnMatch = Not blnMatch

                                    'Otherwise, assume default of textual non-case-sensitive textual comparison
                                    Case Else   
                                        blnMatch = StrComp(strFilterValue, strActualValue, 1) = 0

                                End Select
                        End Select

                        If Not blnMatch Then
                            Exit For
                        End If
                    Next
                End If

                If blnMatch Then
                    If intMatchedEvents > UBound(arrMatchedEvents) Then
                        ReDim Preserve arrMatchedEvents(intMatchedEvents - 1 + 10)  ' Grow by 10
                    End If

                    Set arrMatchedEvents(intMatchedEvents) = objClinEvent

                    intMatchedEvents = intMatchedEvents + 1

                    If blnMatchOneEventOnly Then						
                        Exit For
                    End If
                End If
            Next

            If Not blnMatchOneEventOnly Then
                blnMatch = (intMatchedEvents > 0)
            End If

            ReDim Preserve arrMatchedEvents(intMatchedEvents - 1)

        End If       
    End If 

    Set objFilter = Nothing
    FilterMatch = blnMatch

End Function

Function LoadXMLFromString(strProtocolXML)  'As MSXML2.DOMDocument40
'
' Description:  Loads an XML DOM Document using XML passed in as a string.
' Parameters:   strProtocolXML - the XML to be converted into a DOM document
' Returns:      The corresponding DOM document object.
'

    Dim objProtocolXML      'As Object
    Dim strErrorMsg         'As String

    'Initialise objects
    strErrorMsg = ""
    Set objProtocolXML = Nothing

    'Trap errors
    Err.Clear
    On Error Resume Next

    'Create the protocol XML object
    If Len(strErrorMsg) = 0 Then
        Set objProtocolXML = CreateObject("MSXML2.DOMDocument.4.0")
        If Err.Number <> 0 Then
            strErrorMsg = "Unable to load referral XML. Initialise of DOM object failed." & vbCrLf & _
                            "Please check that the Gateway support has been installed correctly on your machine."
        End If
    End If

    'Load & parse the protocol XML
    If Len(strErrorMsg) = 0 Then
        objProtocolXML.Async = False
        objProtocolXML.ValidateOnParse = True

        ' The protocol XML may or may not have a processing instruction, but it works either way
        objProtocolXML.LoadXML(strProtocolXML)

        DebugWrite "Loading XML:" & vbCrLf & strProtocolXML
        If Err.Number <> 0 Then
            strErrorMsg = "Unable to load referral XML: " & vbCrLf & _
                            Err.Description & vbCrLf & _
                            "Please contact Gateway support."
        ElseIf objProtocolXML.ParseError.ErrorCode <> 0 Then
            strErrorMsg = "Unable to parse referral XML: " & vbCrLf & _
                            objProtocolXML.ParseError.Reason & vbCrLf & _
                            "Please contact Gateway support."
        End If
    End If

    On Error Goto 0

    'Output any parsing / loading errors
    If Len(strErrorMsg) > 0 Then
        MsgBox strErrorMsg, , "SCI Protocol Verification Tool"
        Set objProtocolXML = Nothing
    End If

    Set LoadXMLFromString = objProtocolXML

End Function

Function MatchReferenceData(intRefData, strMatch, strDefault)   'As String
'
' Description:  Checks Gpass reference data for a particular string, tries to match it, and 
'               uses a default if not found.
' Parameters:   intRefData - the type of reference data to search for
'               strMatch - the string to search for
'               strDefault - the default to apply if a match isn't found
'                            this should always be valid within reference data
' Returns:      The valid (according to reference data) string to use.
'
    Dim blnMatch        'As Boolean
    Dim colValues       'As Collection
    Dim strValue        'As String

    'Search reference data for the match value
    blnMatch = False
    Set colValues = g_objGpassApp.ReferenceData(intRefData)
    If Not colValues Is Nothing Then
        For Each strValue In colValues
            If (Trim(LCase(strValue)) = Trim(LCase(strMatch))) Then
                blnMatch = True
                Exit For
            End If
        Next
    End If

    'If we found a match, use it, otherwise use the default
    If blnMatch Then
        MatchReferenceData = strMatch
    Else
        MatchReferenceData = strDefault
    End If

End Function

Sub GetReferralFromInfoForm(objReferral, objForm)
'
' Description:  Goes through a form, attempting to set relevant Referral object properties
' Parameters:   objReferral - the Referral object to populate
'               objForm - the DHTML form
'

    Dim objElement     'As DHTML.input

    Set objReferral = New Referral
    For Each objElement In objForm.Elements
        Call objReferral.LetProperty(objElement.ID, objForm(objElement.ID).Value)
    Next    
    Set objElement = Nothing

End Sub

Function GetGpassInstance()     'As NewGpass.Application
'
' Description:  Checks to see if Gpass is running & returns the active application object.
' Parameters:   None
' Returns:      The application object where active, otherwise Nothing.
'

    Dim objActiveApp    'As NewGpass.Application

    Set objActiveApp = Nothing

    On Error Resume Next

    Err.Clear

    'Get the Gpass application object
    Set objActiveApp = GetObject(, "NewGpass.Application")

    'Check for an error
    If Err.Number <> 0 Then
        Set objActiveApp = Nothing
    End If

    On Error Goto 0

    Set GetGpassInstance = objActiveApp

End Function

Function SaveReferralToGpass(strGPName, strPatKey, strGPCode, _
                       	strComment, strRefNature, strPriority, _
                       	strProvider, strRefType, strReferredTo, _
                       	strSpecialityText, strAttType, strErrMessage)	'As Long					

    Dim objGPASS                'As NewGpass.Application
    Dim objReferral             'As Referral
    Dim objEncounter            'As NewGpass.Encounter
    Dim objPat                  'As NewGpass.Patient
    Dim objClinician            'As NewGpass.Clinician
    Dim objTempClinician        'As NewGpass.Clinician

    Dim colClinicians           'As NewGpass.GenericCollection
    Dim colHospitals            'As NewGpass.GenericCollection
    Dim colReferralTypes        'As NewGpass.GenericCollection

    Dim strHospital             'As Variant 'must be variant
    Dim strReferralType         'As Variant 'must be variant
    Dim blnFoundHospital        'As Boolean
    Dim blnFoundReferralType    'As Boolean

    Dim strErrAction            'As String
    Dim strTempGPCode           'As String
    Dim blnFound

    SaveReferralToGpass = 0
    strErrMessage = ""
    strErrAction = ""

    'Attendence Type
    Select Case UCase(strAttType)
        Case "FIRST"
            strAttType = "1"
        Case "SECOND"
            strAttType = "2"
        Case "SUBSEQUENT"
            strAttType = "3"
        Case Else
            'default value
            strAttType = "1"
    End Select

    'Priority - GPASS and Secondary Care use different terms so both are supported here.
    Select Case UCase(strPriority)
        Case "ROUTINE", "LOW"
            strPriority = "0"
        Case "SOON", "MEDIUM"
            strPriority = "1"
        Case "URGENT", "HIGH"
            strPriority = "2"
        Case Else
            'default value
            strPriority = "0"
    End Select

    'Referral To
    Select Case UCase(strReferredTo)
        Case "PRIVATE"
            strReferredTo = "Private"
        Case "OTHER"
            strReferredTo = "Other"
        Case Else
            'default value (normally always "NHS")
            strReferredTo = "NHS"
    End Select

    'Create Gpass application object
    Set objGPASS = CreateObject("NewGpass.Application")

    If (objGPASS Is Nothing) Then
        SaveReferralToGpass = -1
        strErrMessage = "Cannot create GPASS 'Application' object"
        err.clear
    Else
        'create a new encounter
        Set objEncounter = New Encounter
        If (Not objEncounter Is Nothing) Then
            'get patient object
            Set objPat = objGPASS.GetObject(strPatKey)

            If (Not objPat Is Nothing) Then
                'link patient to encounter
                call objEncounter.LetProperty("Patient", objPat)
                if Err.Number <> 0 then
                    strErrMessage = Err.Description
                    err.clear
                end if					
            Else
                SaveReferralToGpass = -1
                strErrMessage = "Invalid 'Patient Key' parameter"
            End If

            'create clinician object
            Set objClinician = Nothing
            Set colClinicians = objGPASS.Clinicians

            If (Not colClinicians Is Nothing) Then

                strGPCode = Trim(strGPCode)

                If Len(strGPCode) > 0 Then
                    'Remove leading cypher codes if exists from each GPCode (e.g. 'E12345' = '12345')
                    'from referral GPCode
                    If (Not IsNumeric(Left(strGPCode, 1))) Then
                        strGPCode = Mid(strGPCode, 2)
                    End If
                End If

                'try and match the gp passed in by GP Code or GP Name
                blnFound = false

                'try and match on GP Code
                For Each objTempClinician In colClinicians

                    'Remove leading cypher codes if exists from GPASS clinician GPCode
                    strTempGPCode = objTempClinician.GPCode
                    If (Not IsNumeric(Left(strTempGPCode, 1))) Then
                        strTempGPCode = Mid(strTempGPCode, 2)
                    End If
                    'now try to match
                    If (strTempGPCode = strGPCode) Then
                        Set objClinician = objTempClinician
                        blnFound = true
                        Exit For
                    End If

                Next

                If blnFound = false then
                    'try and match on GP Name
                    For Each objTempClinician In colClinicians
                        If Trim(UCase(objTempClinician.Name)) = Trim(UCase(strGPName)) Then
                            Set objClinician = objTempClinician
                            Exit For
	                    End If
                    Next
                End If
            Else
                SaveReferralToGpass = -1
                strErrMessage = "Failed to create a GPASS 'colClinicians' object"
            End If

            If (Not objClinician Is Nothing) Then
                call objEncounter.LetProperty("Clinician", objClinician)
            Else
                'clinical not found
                SaveReferralToGpass = -1
                strErrMessage = "Referring GP '" & strGPName & " with GP code '" & strGPCode & "' not found within GPASS."
            End If

            If (Err.Number <> 0) Then
	            strErrAction = "Unexpected error while searching for referring GP details within GPASS."
	            strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	            SaveReferralToGpass = -1
            End If

            'encounter date - todays date
            call objEncounter.LetProperty("EncounterDate", Now())
            If (Err.Number <> 0) Then
	            strErrAction = "Unexpected error while setting 'Encounter.EncounterDate' within GPASS."
	            strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	            SaveReferralToGpass = -1
            End If

            'comment
            call objEncounter.LetProperty("Comment", "SCI Electronic Referral")
            If (Err.Number <> 0) Then
	            strErrAction = "Unexpected error while setting 'Encounter.Comment' within GPASS."
	            strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	            SaveReferralToGpass = -1
            End If

            'if no errors yet...
            If (SaveReferralToGpass = 0) Then
                'save the encounter
                call objEncounter.DoMethod("Save")
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while saving the GPASS Encounter."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'create referral object
                Set objReferral = New Referral

                'referral date
                call objReferral.LetProperty("Date", Now())
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while setting the 'Referral.AppointmentDate' within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'nature
                call objReferral.LetProperty("Nature", strRefNature)
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while setting the 'Referral.Nature' within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'priority
                call objReferral.LetProperty("Priority", strPriority)
                If (Err.Number <> 0) Then
                    strErrAction = "Unexpected error while setting the 'Referral.Priority' within GPASS."
                    strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
                    SaveReferralToGpass = -1
                End If

                'authorising clinician
                If (Not objClinician Is Nothing) Then
                    call objReferral.LetProperty("AuthorisingClinician", objClinician)

                    If (Err.Number <> 0) Then
	                    strErrAction = "Unexpected error while setting the 'Referral.AuthorisingClinician' within GPASS."
	                    strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                    msgbox strErrMessage, , "SCI Protocol Verification Tool"
	                    SaveReferralToGpass = -1
                    End If
                End If

                'Provider - match hospital passed in
                blnFoundHospital = False
                Set colHospitals = objGPASS.ReferenceData(refdReferralProviders)
                If (Err.Number <> 0) Then
                    strErrAction = "Failed to create a GPASS 'ReferenceData' object."
                    strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
                    SaveReferralToGpass = -1
                End If

                If (Not colHospitals Is Nothing) Then
	                For Each strHospital In colHospitals
                        If (Trim(UCase(strHospital)) = Trim(UCase(strProvider))) Then
                            blnFoundHospital = True
                            Exit For
	                    End If
	                Next
                    If (Not blnFoundHospital) Then
                        'use default "SCI Hospital"
                        strHospital = "SCI Hospital"
                        strComment = strComment & "- Actual Hospital: " & strProvider
                    End If
                    'provider
                    call objReferral.LetProperty("Provider", strHospital)
                    If (Err.Number <> 0) Then
	                    strErrAction = "Unexpected error while setting the 'Referral.Provider' within GPASS."
	                    strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                    SaveReferralToGpass = -1
                    End If
                Else
	                SaveReferralToGpass = -1
	                strErrMessage = "Failed to create a GPASS 'ReferenceData' object"
                End If

                'Referral Type - match refType passed in
                blnFoundReferralType = False
                Set colReferralTypes = objGPASS.ReferenceData(refdReferralTypes)
                If (Not colReferralTypes Is Nothing) Then
                    For Each strReferralType In colReferralTypes
	                    If (Trim(UCase(Replace(strRefType, "_", " "))) = Trim(UCase(strReferralType))) Then
		                    blnFoundReferralType = True
		                    Exit For
	                    End If
                    Next
                    If (Not blnFoundReferralType) Then
	                    'use default "Out Patient"
	                    strReferralType = "Out Patient"
                    End If

                    'Referral Type
                    call objReferral.LetProperty("ReferralType", strReferralType)
                    If (Err.Number <> 0) Then
	                    strErrAction = "Unexpected error while setting the 'Referral.ReferralType' within GPASS."
	                    strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                    SaveReferralToGpass = -1
                    End If
                Else
	                SaveReferralToGpass = -1
	                strErrMessage = "Unexpected Error. Failed to create a GPASS 'ReferenceData' object"
                End If

                'Comment
                call objReferral.LetProperty("Comment", strComment)
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while setting the 'Referral.Comment' within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'Referral To
                call objReferral.LetProperty("ReferredTo", strReferredTo)
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while setting the 'Referral.ReferredTo' within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'Specialty
                call objReferral.LetProperty("Speciality", strSpecialityText)
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while setting the 'Referral.Speciality' within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'Attendance Type
                call objReferral.LetProperty("AttendanceType", strAttType)
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while setting the 'Referral.AttendanceType' within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'Link referral to encounter
                call objReferral.LetProperty("Encounter", objEncounter.getProperty(BASE_OBJECT))
                If (Err.Number <> 0) Then
	                strErrAction = "Unexpected error while linking the Referral to the Encounter within GPASS."
	                strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                SaveReferralToGpass = -1
                End If

                'if no errors yet...
                If (SaveReferralToGpass = 0) Then

                    'save referral
                    objReferral.doMethod("Save")
                    If (Err.Number <> 0) Then
	                    strErrAction = "Unexpected error while saving the Referral within GPASS."
	                    strErrMessage = strErrAction & Chr(10) & "Error: " & Err.Number & Chr(10) & "Description: " & Err.Description & Chr(10) & "Source: " & Err.Source
	                    SaveReferralToGpass = -1
                    End If
                End If
            End If

        Else
            SaveReferralToGpass = -1
            strErrMessage = "Failed to create a GPASS 'Encounter' object"
        End If

    End If

    If (Err.Number <> 0) Then
        'we have errors
        If (strErrMessage = "") Then
	        SaveReferralToGpass = Err.Number
	        strErrMessage = "Unexpected error whilst saving the referral in GPASS." & vbCrLf & vbCrLf & "Err Description: " & Err.Description & ". Err Source: " & Err.Source
        End If
    Else
        If (SaveReferralToGpass = 0) Then
            strErrMessage = "The 'Referral' record has been added to GPASS."
        End If
    End If

    If (SaveReferralToGpass <> 0) Then
        strErrMessage = "Error importing referral into Gpass" & vbCrLf & vbCrLf & strErrMessage & vbCrLf & vbCrLf & "The referral has been sent to the recipient but" & vbCrLf & "the 'Referral' record within GPASS could not be saved."
    End If

    'kill objects
    Set objTempClinician = Nothing
    Set objClinician = Nothing
    Set objTempClinician = Nothing
    Set colClinicians = Nothing
    Set colHospitals = Nothing
    Set colReferralTypes = Nothing
    Set objPat = Nothing
    Set objReferral = Nothing
    Set objEncounter = Nothing
    Set objGPASS = Nothing

End Function

Sub BuildClinicalEventArrays(colClinicalEvents, arrReadCodeEvents, arrScreeningEvents, arrDrugs, _
                ByVal intNoCurrEncRCEs, ByVal intNoCurrEncSEs, ByVal intNoCurrEncPDs)
    Dim intRceCount
    Dim intSeCount
    Dim intDrugCount
    Dim objClinicalEvent

    If IsArray(arrReadCodeEvents) Then
        intRCECount = UBound(arrReadCodeEvents) + 1
        ReDim Preserve arrReadCodeEvents(intRCECount - 1 + 20)
    Else
        intRCECount = 0
        ReDim arrReadCodeEvents(19)
    End If

    If IsArray(arrScreeningEvents) Then
        intSECount = UBound(arrScreeningEvents) + 1
        ReDim Preserve arrScreeningEvents(intSECount - 1 + 20)
    Else
        intSECount = 0
        ReDim arrScreeningEvents(19)
    End If

    If IsArray(arrDrugs) Then
        intDrugCount = UBound(arrDrugs) + 1
        ReDim Preserve arrDrugs(intDrugCount - 1 + 20)
    Else
        intDrugCount = 0
        ReDim arrDrugs(19)
    End If

    For Each objClinicalEvent In colClinicalEvents
        AddClinicalEventToArray objClinicalEvent, arrReadCodeEvents, arrScreeningEvents, arrDrugs, intRCECount, intSECount, intDrugCount, intNoCurrEncRCEs, intNoCurrEncSEs, intNoCurrEncPDs
    Next

    ' Shrink arrays to actual number of elements
    ReDim Preserve arrReadCodeEvents(intRCECount - 1)
    ReDim Preserve arrScreeningEvents(intSECount - 1)
    ReDim Preserve arrDrugs(intDrugCount - 1)

End Sub

Function EventAlreadyInArray(objProtocolObject, arrEventArray, ByVal intNoCEElements)
'
' Description:  This function checks whether the passed object is already in the passed event array.
' Parameters:   objProtocolObject - A protocolObject for the Event
'               arrEventArray - The array to check
'               intNoCEElements - The number of current encounter elements in arrEventArray
'                   NB: This is NOT the total number of elements but the number of elements within
'                   the current encounter. This number is used for performance reasons as it is
'                   likely to be a small number or zero even if the patient has lots of events and
'                   will therefore reduce the number of comparisons.
' Returns:      True if the object is already in the array, False otherwise
'
    Dim intIndex
    Dim strKey

    ' Default to not existing and if there are no current encounter events then
    ' nothing to do
    EventAlreadyInArray = False
    If intNoCEElements < 1 or Not IsArray(arrEventArray) then Exit Function

    strKey = objProtocolObject.GetProperty("Key")

    ' Safety Check
    if intNoCEElements > UBound(arrEventArray) Then
        intNoCEElements = UBound(arrEventArray)
    End If

    For intIndex = 0 to intNoCEElements - 1
        if arrEventArray(intIndex).GetProperty("Key") = strKey Then
	        EventAlreadyInArray = True
	        Exit For
        End If
    Next

End Function

Sub AddClinicalEventToArray(objClinicalEvent, arrReadCodeEvents, arrScreeningEvents, arrDrugs, _
                        intRCECount, intSECount, intDrugCount, _
                        ByVal intNoCurrEncRCEs, ByVal intNoCurrEncSEs, ByVal intNoCurrEncPDs)
    Dim objProtocolObject
    Dim objCareValue
    Dim objCareEvent
    Dim objCollectionValue

    Set objProtocolObject = New ProtocolObject
    objProtocolObject.Use objClinicalEvent

    Select Case TypeName(objClinicalEvent)
        Case "ReadCodeEvent"
            If Not EventAlreadyInArray(objProtocolObject, arrReadCodeEvents, intNoCurrEncRCEs) Then
                If intRCECount > UBound(arrReadCodeEvents) Then
                    ReDim Preserve arrReadCodeEvents(intRCECount - 1 + 10)  ' grow by 10
                End If
                Set arrReadCodeEvents(intRCECount) = objProtocolObject
                intRCECount = intRCECount + 1
            End If

        Case "ScreeningEvent"				
            If Not EventAlreadyInArray(objProtocolObject, arrScreeningEvents, intNoCurrEncSEs) Then
                If intSECount > UBound(arrScreeningEvents) Then
                    ReDim Preserve arrScreeningEvents(intSECount - 1 + 10)  ' grow by 10
                End If
                Set arrScreeningEvents(intSECount) = objProtocolObject
                intSECount = intSECount + 1
            End If

        Case "PrescribedDrug"
            If Not EventAlreadyInArray(objProtocolObject, arrDrugs, intNoCurrEncPDs) Then
                If intDrugCount > UBound(arrDrugs) Then
                    ReDim Preserve arrDrugs(intDrugCount - 1 + 10)  ' grow by 10
                End If
                Set arrDrugs(intDrugCount) = objProtocolObject
                intDrugCount = intDrugCount + 1
            End If

    	Case "CareSession"
            For Each objCareValue in objClinicalEvent.Values
                If TypeName(objCareValue.Value) = "ReadCodeEvent" _
                Or TypeName(objCareValue.Value) = "ScreeningEvent" _
                Or TypeName(objCareValue.Value) = "PrescribedDrug" Then
                    AddClinicalEventToArray objCareValue.Value, arrReadCodeEvents, arrScreeningEvents, arrDrugs, _
                            intRCECount, intSECount, intDrugCount, _
                            intNoCurrEncRCEs, intNoCurrEncSEs, intNoCurrEncPDs
                Else
                    If TypeName(objCareValue.Value) = "ListCollection" then
                        For Each objCollectionValue in objCareValue.Value
                            If TypeName(objCollectionValue) = "ReadCodeEvent" _
                            Or TypeName(objCollectionValue) = "ScreeningEvent" _
                            Or TypeName(objCollectionValue) = "PrescribedDrug" Then
                                AddClinicalEventToArray objCollectionValue, arrReadCodeEvents, arrScreeningEvents, arrDrugs, _
                                        intRCECount, intSECount, intDrugCount, _
                                        intNoCurrEncRCEs, intNoCurrEncSEs, intNoCurrEncPDs
                            End If
                        Next
                    End if
                End If
            Next
    End Select
End Sub

Sub InitialiseProgressBar(intMax)
    If WindowStillOpen() Then
        If Not m_objClinEventProgMain Is Nothing Then
            m_objClinEventProgMain.style.width = 204 & "px"
            m_objClinEventProgMain.style.border = "thin solid black"
        End If

        If Not m_objClinEventProgDetail Is Nothing Then
            m_objClinEventProgDetail.style.width = "0px"
            m_objClinEventProgDetail.style.backgroundcolor = "lightblue"
        End If
    End If
End Sub 

Sub UpdateProgressBar(intCurrent, intMax)
    If WindowStillOpen() Then
        If Not m_objClinEventProgDetail Is Nothing Then
            m_objClinEventProgDetail.style.width = CStr(intCurrent * PROGRESS_BAR_WIDTH \ intMax) & "px"
        End If

        If Not m_objClinEventProgNumbers Is Nothing Then
            'm_objClinEventProgNumbers.innerHTML = "<b>" & intCurrent & " of " & intMax & "</b>"
            m_objClinEventProgNumbers.innerHTML = "<b>" & intCurrent * 100 \ intMax & "%"
        End If
    End If
End Sub

Sub HideProgressBar()
    If WindowStillOpen() Then
        If Not m_objClinEventProgMain Is Nothing Then
            m_objClinEventProgMain.style.display = "none"
            m_objClinEventProgNumbers.style.display = "none"
        End If
    End If
End Sub

Sub SetPrePopMsg(strMsg)
    If WindowStillOpen() Then
        If Not m_objPrePopMsg Is Nothing Then
            m_objPrePopMsg.innerHTML = strMsg
        End If
    End If
End Sub

Function WindowStillOpen()
    Dim strScreenInstanceName

    strScreenInstanceName = ""
    On Error Resume Next
    strScreenInstanceName = m_objPrePopWin.name
    On Error GoTo 0
    WindowStillOpen = (strScreenInstanceName = m_strPrePopWindowName)
End Function

Sub ClosePrePopMsgWindow()
    On Error Resume Next
    If Not m_objPrePopWin Is Nothing Then
        m_objPrePopWin.close()
    End If
    On Error GoTo 0 
End Sub

Function MatchAndPopulate(objQuestion, arrRCE, arrSE, arrDrug, strID, strType, intCount, intEventType, objItem, blnMatchOnlyOneEvent)						
    Dim blnMatchesFilter
    Dim intMax
    Dim objMatchedEvent
    Dim arrMatchedEvents
    Dim intMatchedEventIndex
    Dim strValue                'As String
    Dim objMatch                'As MSXML2.DOMNode

    intMax = MAX_RECORDS 'the default maximum number of matching records is 500

    'Match the filters of the object specified against the specified clinical event array by intEventType
    Select Case intEventType
        Case EVENTTYPE_READCODE
            If Not IsEmptyArray(arrRCE)  Then
                blnMatchesFilter = FilterMatch(arrRCE, objItem.selectNodes("filters[@type='GPASS']/filter"), intMax, arrMatchedEvents, blnMatchOnlyOneEvent)
            End If
        Case EVENTTYPE_SCREENING
            If Not IsEmptyArray(arrSE)  Then
                blnMatchesFilter = FilterMatch(arrSE, objItem.selectNodes("filters[@type='GPASS']/filter"), intMax, arrMatchedEvents, blnMatchOnlyOneEvent)
            End If
        Case EVENTTYPE_DRUG         
            If Not IsEmptyArray(arrDrug)  Then
                blnMatchesFilter = FilterMatch(arrDrug, objItem.selectNodes("filters[@type='GPASS']/filter"), intMax, arrMatchedEvents, blnMatchOnlyOneEvent)
            End If
    End Select

    'Set the values from the matched events to the HTML elements
    If blnMatchesFilter Then
        Select Case strType
            Case "radio" 
                Call SetHTMLRadioValue(document.forms(FORM_ID).all(strID), objItem.getAttribute("value") & "")
            Case "select"
                Call SetHTMLSelectValue(document.forms(FORM_ID).all(strID), objItem.getAttribute("value")& "")
            Case "text", "textarea"
                document.forms(FORM_ID).all(strID).value = arrMatchedEvents(0).GetProperty(objQuestion.getAttribute("propertyOld") & "")
            Case "check"
                If InStr(document.forms(FORM_ID).all(strID).outerHTML, "checked=true") = 0 Then
                    document.forms(FORM_ID).all(strID).outerHTML = Replace(document.forms(FORM_ID).all(strID).outerHTML, ">", " checked=true>")
                End If
            Case "table", "dynaTable"
                'the returned array "arrMatchedEvents" holds all matching clinical events for this (dyna)table
                intCount = 0
                For Each objMatchedEvent In arrMatchedEvents
                    If intCount <= intMax Then
                        If strType = "dynaTable" Then
                            ' Add a row in the dynatable
                            intCount = AddRow(strID) - 1
                        End If

                        ' Add the details to the cells of the added row				
                        PopulateTableRow objQuestion, strID, intCount, objMatchedEvent
                    Else
                        Exit For
                    End If
                    intCount = intCount + 1					
                Next
        End Select
    End If
End Function

Function IsEmptyArray(arrEvents)
    IsEmptyArray = (LBound(arrEvents) > UBound(arrEvents))
End Function

Function buildFilterArray(objFilterList)
    Dim arrFilter
    Dim objFilter
    Dim intNoOfFilter
    Dim intFilterIndex
        	
    intFilterIndex = 0
    intNoOfFilter = objFilterList.Length

    Redim arrFilter(0)
        
    If intNoOfFilter > 0 Then
        Redim arrFilter(intNoOfFilter, 2)
            
        For Each objFilter in objFilterList
            arrFilter(intFilterIndex, FILTER_ATTRIB_PROPERTY) = objFilter.getAttribute("property") & ""
            arrFilter(intFilterIndex, FILTER_ATTRIB_COMPARISON) = objFilter.getAttribute("comparison") & ""
            arrFilter(intFilterIndex, FILTER_ATTRIB_VALUE) = objFilter.getAttribute("value") & ""
                    
            intFilterIndex = intFilterIndex + 1
        Next
    End If			
    buildFilterArray = arrFilter
End Function

