'============================================================================================
' File name:        gpassPatient.vbs
' Created:          17-Jun-2002
' Author:           D. Allen-Williams
' Description:      This provides methods for extracting supported Patient properties used
'                   by protocol definitions.  In particular, it assumes a bolt on to the standard
'                   Patient object exposed by the NewGpass API.
' Dependencies:     
' Issues:           
'============================================================================================

Option Explicit

Class Patient

    'Decorated object
    Private m_objProtocolObj       'As ProtocolObject

    'Custom additional properties that we support
    Private m_strAddress           'As String
    Private m_strBloodPressure     'As String
    Private m_strBMI               'As String
    
    '---------------------------------------------
    Private Sub Class_Initialize()
        Set m_objProtocolObj = New ProtocolObject
    End Sub
    
    '---------------------------------------------
    Private Sub Class_Terminate()
        Set m_objProtocolObj = Nothing
    End Sub
    
    '---------------------------------------------
    Public Sub Use(objBaseObject)
        Call m_objProtocolObj.Use(objBaseObject)
        Call Initialise()
    End Sub
    
    '---------------------------------------------
    Public Sub Initialise()
        'Initialise custom fields (address)
        'Initialise Address custom field
        m_strAddress = Trim(m_objProtocolObj.GetProperty("Address1")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address2")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address3")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address4")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address5")) 
        Do While InStr(m_strAddress, vbCrLf & vbCrLf) > 0 
            m_strAddress = Replace (m_strAddress, vbCrLf & vbCrLf, vbCrLf)
        Loop        
        If InStr(m_strAddress, vbCrLf) = 1 Then
            m_strAddress = Mid(m_strAddress, 3)
        End If
        If m_objProtocolObj.GetProperty("Systolic") < 0 Then
            m_strBloodPressure = "[unknown] / "
        Else
            m_strBloodPressure = CStr(m_objProtocolObj.GetProperty("Systolic")) & " / "
        End If
        If m_objProtocolObj.GetProperty("Diastolic") < 0 Then
            m_strBloodPressure = m_strBloodPressure & "[unknown]"
        Else
            m_strBloodPressure = m_strBloodPressure & CStr(m_objProtocolObj.GetProperty("Diastolic"))
        End If
        If m_objProtocolObj.GetProperty("Height") > 0 And _
            m_objProtocolObj.GetProperty("Weight") > 0 Then
            m_strBMI = FormatNumber(m_objProtocolObj.GetProperty("Weight") / m_objProtocolObj.GetProperty("Height") / m_objProtocolObj.GetProperty("Height"), 2, -1)
        Else
            m_strBMI = ""
        End If
        
    End Sub

    '---------------------------------------------
    Public Function GetProperty(strProperty)    'As Variant

        Dim varValue          'As Variant
        
        varValue = ""
        Select Case strProperty
        
            'Check for one of our custom properties
            Case "Address"                   : varValue = m_strAddress
            Case "BloodPressure"             : varValue = m_strBloodPressure
            Case "BodyMassIndex"             : varValue = m_strBMI
            
            'Handle objects non-generically, since we need to set an object reference
            Case "RegisteredClinician", "SeenByClinician", "ClinicalEvents" ', any other objects
                Set GetProperty = m_objProtocolObj.GetProperty(strProperty)
                Exit Function
            
            'Otherwise, assume it comes from the base object, 
            'overriding errors (hence provide the blank default)
            Case Else
                varValue = m_objProtocolObj.GetProperty(strProperty)
                        
        End Select
        GetProperty = varValue
        
    End Function
    
    '---------------------------------------------
    Public Sub LetProperty(strProperty, varValue)    

        Dim objPatient      'As NewGpass.Patient
        
        Select Case strProperty
        
            'Check for the only property that currently might be set by the app
            Case "Key"                       
                Set objPatient = g_objGpassApp.GetObject(varValue)
                If Not objPatient Is Nothing Then      
                    Call Use(objPatient)       'initialise our ProtocolObject
                    Set objPatient = Nothing
                End If                
                
            'Otherwise, assume it's a property of the base object
            Case Else
                Call m_objProtocolObj.LetProperty(strProperty, varValue)

        End Select
        
    End Sub
    
End Class


