'============================================================================================
' File name:        gpassPremises.vbs
' Created:          17-Jun-2002
' Author:           D. Allen-Williams
' Description:      This provides methods for extracting supported Premises properties used
'                   by protocol definitions.  In particular, it assumes a bolt on to the standard
'                   Premises object exposed by the NewGpass API.
' Dependencies:     
' Issues:           
'============================================================================================

Option Explicit

Class Premises

    'Decorated object
    Private m_objProtocolObj       'As ProtocolObject

    'Custom additional properties that we support
    Private m_strAddress           'As String
    Private m_strTelephone         'As String
    
    '---------------------------------------------
    Private Sub Class_Initialize()
        Dim objPremises             'As NewGpass.Premises
        
        Set m_objProtocolObj = New ProtocolObject
        
        Set objPremises = g_objGpassApp.Premises.Item(1)
            
        Call Use(objPremises)
        
        Set objPremises = Nothing        
    End Sub
    
    '---------------------------------------------
    Private Sub Class_Terminate()
        Set m_objProtocolObj = Nothing
    End Sub
    
    '---------------------------------------------
    Public Sub Use(objBaseObject)
        Call m_objProtocolObj.Use(objBaseObject)
        Call Initialise()
    End Sub
    
    '---------------------------------------------
    Public Sub Initialise()
        'Initialise Address custom field
        m_strAddress = Trim(m_objProtocolObj.GetProperty("Address1")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address2")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address3")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address4")) & vbCrLf & _
                       Trim(m_objProtocolObj.GetProperty("Address5")) 
        Do While InStr(m_strAddress, vbCrLf & vbCrLf) > 0 
            m_strAddress = Replace (m_strAddress, vbCrLf & vbCrLf, vbCrLf)
        Loop
        If InStr(m_strAddress, vbCrLf) = 1 Then
            m_strAddress = Mid(m_strAddress, 3)
        End If

        'Initialise Telephone custom field
        m_strTelephone = m_objProtocolObj.GetProperty("Telephone1") 
        If Len(m_strTelephone) = 0 Then
            m_strTelephone = m_objProtocolObj.GetProperty("Telephone2") 
        End If
    End Sub
    
    '---------------------------------------------
    Public Function GetProperty(strProperty)    'As Variant

        Dim varValue          'As Variant
        
        varValue = ""
        Select Case strProperty
        
            'Check for one of our custom properties
            Case "Address"                   : varValue = m_strAddress
            Case "Telephone"                 : varValue = m_strTelephone
            
            'Otherwise, assume it comes from the base object, 
            'overriding errors (hence provide the blank default)
            Case Else
                varValue = m_objProtocolObj.GetProperty(strProperty)
                        
        End Select
        GetProperty = varValue
        
    End Function
    
    '---------------------------------------------
    Public Sub LetProperty(strProperty, varValue)    

        Dim objPremises      'As NewGpass.Patient
        
        Select Case strProperty
        
            'Check for one of our custom properties
            Case "Address"                   : m_strAddress = CStr(varValue)
            Case "Telephone"                 : m_strTelephone = CStr(varValue)

            'Check for the only property that currently might be set by the app
            Case "Key"                       
                Set objPremises = g_objGpassApp.GetObject(varValue)
                If Not objPremises Is Nothing Then     
                    Call Use(objPremises)       'initialise our ProtocolObject
                    Set objPremises = Nothing
                End If                
                
            'Otherwise, assume it's a property of the base object
            Case Else
                Call m_objProtocolObj.LetProperty(strProperty, varValue)

        End Select
        
    End Sub
    
End Class

