Option Explicit
'============================================================================================
' File name:    gpassReferral.vbs
' Description:  This provides methods for extracting supported Referral properties used
'               by protocol definitions.  In particular, it assumes a bolt on to the standard
'               Referral object exposed by the NewGpass API.
'============================================================================================

Const refdReferralTypes     = 19
Const refdReferralProviders = 20
Const refdPHCTMemberType    = 45

Const DEF_PROVIDER          = "SCI Hospital"
Const DEF_ROLE              = "Gp Principal"
Const DEF_TYPE              = "Out Patient"

Class Referral

    'Decorated object
    Private m_objProtocolObj                'As ProtocolObject

    'Custom additional properties that we support
    Private m_strAuthorisingClinicianName   'As String
    Private m_strConsultantForename         'As String
    Private m_strConsultantGrade            'As String
    Private m_strConsultantID               'As String
    Private m_strConsultantSurname          'As String
    Private m_strHospitalAddress            'As String
    Private m_strHospitalID                 'As String
    Private m_strHospitalName               'As String
    Private m_strHospitalTelephone          'As String
    Private m_strProtocol                   'As String

    Private Sub Class_Initialize()       
        Dim objRef      'As NewGpass.Referral

        Set m_objProtocolObj = New ProtocolObject
        Set objRef = CreateObject("NewGpass.Referral")
        
        If Not objRef Is Nothing Then
            Call m_objProtocolObj.Use(objRef)
            'Set correct default priority
            Call LetProperty("Priority", 0)
        End If
        
        Set objRef = Nothing
    End Sub

    Private Sub Class_Terminate()
        Set m_objProtocolObj = Nothing
    End Sub

    Public Sub Use(objBaseObject)
        Call m_objProtocolObj.Use(objBaseObject)
    End Sub

    Public Function GetProperty(strProperty)    'As Variant

        Dim varValue          'As Variant

        varValue = ""
        Select Case strProperty

            'Check for one of our custom properties
            Case "AuthorisingClinicianName"  : varValue = m_strAuthorisingClinicianName
            Case "ConsultantForename"        : varValue = m_strConsultantForename
            Case "ConsultantGrade"           : varValue = m_strConsultantGrade
            Case "ConsultantID"              : varValue = m_strConsultantID
            Case "ConsultantSurname"         : varValue = m_strConsultantSurname
            Case "HospitalAddress"           : varValue = m_strHospitalAddress
            Case "HospitalID"                : varValue = m_strHospitalID
            Case "HospitalName"              : varValue = m_strHospitalName
            Case "HospitalTelephone"         : varValue = m_strHospitalTelephone
            Case "Protocol"                  : varValue = m_strProtocol

            'Cater for Gpass' spurious spelling of Specialty (Speciality)
            Case "Specialty"                 : varValue = m_objProtocolObj.GetProperty("Speciality")

            'Handle objects non-generically, since we need to set an object reference
            Case "AuthorisingClinician" ', any other objects
                Set GetProperty = m_objProtocolObj.GetProperty(strProperty)
                Exit Function

            'Otherwise, assume it comes from the base object, 
            'overriding errors (hence provide the blank default)
            Case Else
                varValue = m_objProtocolObj.GetProperty(strProperty)                        
        End Select

        GetProperty = varValue
        
    End Function
    
    Public Sub LetProperty(strProperty, varValue)

        Dim objClinician        'As NewGpass.Clinician
        Dim colClinicians       'As Collection

        Select Case strProperty
        
            'Check for one of our custom properties
            Case "ConsultantForename"       : m_strConsultantForename = CStr(varValue)
            Case "ConsultantGrade"          : m_strConsultantGrade = CStr(varValue)
            Case "ConsultantID"             : m_strConsultantID = CStr(varValue)
            Case "ConsultantSurname"        : m_strConsultantSurname = CStr(varValue)
            Case "HospitalAddress"          : m_strHospitalAddress = CStr(varValue)
            Case "HospitalID"               : m_strHospitalID = CStr(varValue)
            Case "HospitalName"             : m_strHospitalName = CStr(varValue)
            Case "HospitalTelephone"        : m_strHospitalTelephone = CStr(varValue)
            Case "Protocol"                 : m_strProtocol = CStr(varValue)

            'cater for property set from the hidden info form
            'where the field name cannot be the same as the property name
            'to avoid duplicate field names on the page.
            Case "ProtocolName"             : m_strProtocol = CStr(varValue)

            'Cater for Gpass' spurious spelling of Specialty
            Case "Specialty"                : Call m_objProtocolObj.LetProperty("Speciality", CStr(varValue))

            'cater for property set from the hidden info form
            'where the field name cannot be the same as the property name
            'to avoid duplicate field names on the page.
            Case "SpecialtyName"            : Call m_objProtocolObj.LetProperty("Speciality", CStr(varValue))

            Case "AuthorisingClinician", "AuthorisingClinicianName"
                If lcase(typeName(varValue)) = "clinician" Then
                    m_strAuthorisingClinicianName = varValue.name
                    Call m_objProtocolObj.LetProperty("AuthorisingClinician", varValue)
                ElseIf lcase(typeName(varValue)) = "string" Then
                    m_strAuthorisingClinicianName = varValue

                    Set colClinicians = g_objGpassApp.Clinicians()

                    'Now search for the appropriate clinician
                    For Each objClinician In colClinicians
                        If Trim(LCase(objClinician.Name)) = Trim(LCase(m_strAuthorisingClinicianName)) Then
                            Call m_objProtocolObj.LetProperty("AuthorisingClinician", objClinician)
                            Exit For
                        End If
                    Next
                    Set objClinician = Nothing
                    Set colClinicians = Nothing
                End If
            	
            'Handle cases where there is a list of valid items
            Case "Provider"
                Call m_objProtocolObj.LetProperty("Provider", _
                                                    MatchReferenceData(refdReferralProviders, _
                                                                        varValue, _
                                                                        DEF_PROVIDER))
            Case "ReferralType"
                Call m_objProtocolObj.LetProperty("ReferralType", _
                                                    MatchReferenceData(refdReferralTypes, _
                                                                        varValue, _
                                                                        DEF_TYPE))
            Case "ReferringRole"
                Call m_objProtocolObj.LetProperty("ReferringRole", _
                                                    MatchReferenceData(refdPHCTMemberType, _
                                                                        varValue, _
                                                                        DEF_ROLE))

            'Otherwise, assume it's a property of the base object
            Case Else
                On Error Resume Next
                Call m_objProtocolObj.LetProperty(strProperty, varValue)
                On Error Goto 0
        End Select

    End Sub

    Public Sub DoMethod(strMethodName)
        Call m_objProtocolObj.doMethod(strMethodName)
    End Sub
    
End Class