Option Explicit
'============================================================================================
' File name:    viewprotocol.vbs
' Description:  Supporting script for the viewprotocol.asp page.
'============================================================================================

Const COMMENT_MARKER = "//"
Const DYNATABLE_CLASS_NAME = "DynaTable"
Const FILE_DELIMITER = "|"

Const CAUTION_TEXT = "//CREATED BY PROTOCOL VERIFICATION TOOL %date. DO NOT MODIFY THIS FILE!"
Const FILE_SECTION_MARKER = "//SECTION BEGIN:"
Const DYNATABLE_SECTION_MARKER = "//DYNATABLE"
Const MSG_SIZE_LIMIT = 100000

Const FORM_ID = "formRef"                       'The main form's HTML tag name
Const GPASS_REPLACE_ELEMENT = "protocolPage"    'The DHTML element that will be repopulated by activeGPASS.ProtocolWriter

Dim g_objGpassApp           'As NewGpass.Application
Dim m_strStatusText         'As String

'validation types
Const CHI_FORMAT_ONLY       = 1
Const CHI_DOB               = 2
Const CHI_SEX               = 3
Const CHI_DOB_SEX           = 4 

'Unique Care Pathway Number
Const DUMMY_PATHWAY_NUMBER = "1ZZ000000051U"

Sub ProtocolOpen()
'
' Description:  Initialises form fields on an opened message.
' Parameters:   None
'
    Dim chiNumber
    Dim sex, dateOfBirth, severity, message, validationType
    Dim instruction
    Dim response
    
    call window.resizeTo(800,720)	' was 800,600 but changed for TT53983

    document.body.style.cursor = "auto"
    
    'Initialise tabs & details if this is a new message.  Within Gateway a new message is one where the message id is 0.
    'Within PVT a new message is one where no dat file has been selected and it is not a response to a request.
    If LCase(Right(document.all("dataEntryXmlPath").value,4)) <> ".dat" And Not document.all("isReply").value Then
        Call InitialiseNewMessage()
    End If
    
    ' Validate CHI number and warn if it is invalid.
    If Not document.forms(FORM_ID)("chi") Is Nothing Then

        ' Pull CHI number out of the form
        chiNumber = document.forms(FORM_ID)("chi").value
        validationType = setValidationType()
        
        ' Validate only if a CHI number has been specified, i.e. pulled out of GP system,
        ' and sex has been specified.
        If chiNumber<>"" Then
            ' Determine the patient's sex; we know that exactly one of male or female
            ' has been selected.
            Select Case validationType
                Case CHI_DOB
                    dateOfBirth = document.all("patient_dob").value
                Case CHI_SEX
                    If document.all("patient_sex_1").checked Then
                        sex = MALE
                    Else
                        sex = FEMALE
                    End If                      
                Case CHI_DOB_SEX
                    If document.all("patient_sex_1").checked Then
                        sex = MALE
                    Else
                        sex = FEMALE
                    End If                                                  
                    dateOfBirth = document.all("patient_dob").value
            End Select
            
            If Not isValidChiNumber(chiNumber, dateOfBirth, sex, validationType, severity, message) Then
                If severity = CHI_ERROR Then
                    If document.forms(FORM_ID)("chi").readonly Then ' Populated from GP system
                        instruction = "Please correct the CHI number in GP system. If a correct CHI number is not available please blank the field until it can be determined. You will then have to start a new message."
                    Else
                        instruction = "If a correct CHI number is not available, please leave the field blank, unless it is mandatory."
                    End If
                    MsgBox "Warning: The patient's CHI number is invalid:" & vbNewLine & vbNewLine & _
                        message & vbNewLine & vbNewLine & instruction, _
                        vbExclamation + vbOKOnly, "SCI Gateway"
                Else
                    instruction = "Please verify the CHI number."
                    If document.forms(FORM_ID)("chi").readonly Then ' Populated from GP system
                        instruction = instruction & " If you believe it is incorrect, please correct it in GP system. You will then have to start a new message."
                    Else
                        instruction = instruction & " If a correct CHI number is not available, please blank the CHI number field, unless it is mandatory."
                    End If

                    response = MsgBox("Warning: The patient's CHI number does not match patient data: " & vbNewLine & vbNewline & _
                        message & vbNewLine & vbNewline & instruction, _
                        vbExclamation + vbOKOnly, "SCI Gateway")
                    ' Allow the user to continue if they're sure
                End If
            End If
        End If
    End If
    
    If Not document.forms(FORM_ID)("hcn") Is Nothing Then
        chiNumber = Replace(document.forms(FORM_ID)("hcn")," ", "")
    End If
    
    If Not document.forms(FORM_ID)("patient_nhs_number") Is Nothing Then
        chiNumber = Replace(document.forms(FORM_ID)("patient_nhs_number")," ", "")
    End If
    
    'TT52566
    'check if there are any controls, if so, initialise the control's fields    
    If Not document.all("controls") Is Nothing Then
        call initialiseControls()
    End If
    
    
    'Some elements must always be read-only.  Do this last to make sure.
    'Test Track 51520 and 50922
    call MakeFieldsReadOnly()

    Call UnserialiseFormElements()
    
End Sub

Sub ProtocolPark()
'
' Description:  Parks form fields to a DAT file.
' Parameters:   None
'
    Dim strFileName     'As String

    strFileName = InputBox("Please enter name of the text file that will contain the parked data. " & vbCrlf & vbCrlf & _
                            "Unless a drive letter is included, the file location will be the same folder as the protocol." & _
                            vbCrlf & vbCrlf & _ 
                            "THE FILENAME YOU SUPPLY WILL HAVE THE .DAT EXTENSION APPLIED AUTOMATICALLY." & _
                            vbCrlf & vbCrlf & _  
                            "Note: In the real Gateway application this dialog box will not appear.", _
                            "Enter file name", _
                            CStr(document.all("dataSavePath").value))

    If Trim(strFileName) <> "" Then
        ' Needs a more elaborate file extension checking algorithm, but to be honest who'll 
        ' try and break this system?
        If UCase(Right(strFileName,4)) <> ".DAT" Then
            strFileName = strFileName & ".DAT"
        End If
        
        ' If we dont have a drive location, get the protocol path so we can save the dat file to the same folder as the protocol
        If InStr(1, strFilename, ":", 1) <> 2 Then
            Dim path
            path = document.getElementById("pvtClientProtocolPath").value
            path = Left(path, InStrRev(path, "\", -1 , 1))
            strFileName = path & strFileName
        End If
            
        If SerialiseFormElements(strFileName) Then
            MsgBox "Message has been parked to " & strFileName & " successfully.", , "SCI Protocol Verification Tool"
            document.all("dataSavePath").value = strFileName
        End If
    End If
End Sub

Sub ProtocolXML()
'
' Description:  Views the protocol XML.
' Parameters:   None
'
    Dim objWin      'As Window
    Dim i
    Dim strXMLPath
    Dim strPostInfo 'As String

    'Transfer the form over to a new window
    Set objWin = window.open("about:blank", "RefXML", "scrollbars, resizable, height=560, width=760, top=20, left=20")        

    While objWin.document.readyState <> "complete"
    Wend

    objWin.document.body.innerHTML = "<div style='display: none'>" & document.forms(FORM_ID).outerHTML & "</div>"

    If Not document.all("pvtServerProtocolPath") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""pvtServerProtocolPath"" id=""pvtServerProtocolPath"" value=""" & document.all("pvtServerProtocolPath").value & """ />"
    End If  	

    If Not document.all("msgtypeid") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""msgtypeid"" id=""msgtypeid"" value=""" & document.all("msgtypeid").value & """ />"
    End If
     
    If Not document.all("targetSchemaString") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""targetSchemaString"" id=""targetSchemaString"" value=""" & document.all("targetSchemaString").value & """ />"
    End If
    
    objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""previewtype"" id=""previewtype"" value=""2""/>"

    strPostInfo = BuildPostInfo()
    objwin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", strPostInfo        
        
    'Set up the form with a previewProviderID, and submit
    objWin.document.forms(FORM_ID).action = "../message/previewletter.asp" 
    objWin.document.forms(FORM_ID).submit()

    Set objWin = Nothing

End Sub

Function SerialiseFormElements(ByVal strFileName)
'
' Description:  Dump contents of form to disk file.
' Parameters:   strFileName - Filename to use
'
    Const ForWriting = 2
        
    Dim objWSH      'As Scripting.FileSystemObject
    Dim objFile     'As Scripting.File
        
    SerialiseFormElements = False

    Set objWSH = CreateObject("Scripting.FileSystemObject")

    If objWSH.FileExists(strFileName) Then
        If MsgBox("The file " & strFileName & " exists." & vbCrLf & "Would you like to overwrite it?", vbYesNo, "SCI Protocol Verification Tool") = vbNo Then
            Set objWSH = Nothing
            Exit Function
        End If
    End If
    Set objFile = objWsh.OpenTextFile (strFileName, ForWriting, True)
        
    objFile.WriteLine Replace(CAUTION_TEXT, "%date", Now())
        
    SerialiseChildElements objFile, "formRef"

    objFile.Close
    Set objWSH = Nothing

    SerialiseFormElements = True
    
End Function

Sub SerialiseChildElements (ByRef objStream, ByVal strElement)
'
' Description:  Write all text-containing elements to file. Will handle tables.
' Parameters:   
'
    on error goto 0
    Dim elParent, elChild
        
    Set elParent = document.getElementById(strElement)
                
    If Not elParent Is Nothing Then
        For Each elChild in elParent.all
            StreamElement objStream, elChild
        Next
    End If
End Sub

Function UnSerialiseFormElements()
'
' Description:  Read contents of disk file and populate the form.
' Parameters:   None
'
    Const ForReading = 1
        
    Dim objWSH              'As Scripting.FileSystemObject
    Dim objFile             'As Scripting.FileSystemObject
    Dim strALineOfText      'As String
    Dim strTableName        'As String
    Dim intTempNumRows      'As Integer
    Dim arrElementData      'As Array
    Dim lBd                 'As Byte
    Dim blnIsNotComment     'As Boolean
    Dim strElementType      'As String, populated from row in file
    Dim strElementName      'As String, populated from row in file
    Dim strElementValue     'As String, populated from row in file
    Dim strFileName         'As String
        
    On Error Resume Next

    'Get the file name to load
    strFileName = ""
    If Not document.all("dataLoadPath") Is Nothing Then
        strFileName = CStr(document.all("dataLoadPath").value)
    End If        

    If Len(strFileName) = 0 Then
        Exit Function
    End If

    Set objWSH = CreateObject("Scripting.FileSystemObject")
    If Err.Number <> 0 Then
        MsgBox "Unable to create ActiveX object to open data file", , "SCI Protocol Verification Tool"
        Exit Function
    End If
        
    'Open file for reading
    Set objFile = objWSH.OpenTextFile (strFileName, ForReading)
    If Err.Number <> 0 Then
        MsgBox "Unable to open " & strFileName & " for reading: " & Err.Description, , "SCI Protocol Verification Tool"
        Exit Function
    End If

    On Error Goto 0

    Do While not objFile.AtEndOfStream

        strALineOfText = objFile.ReadLine()

        ' Have we hit any comments?           
        blnIsNotComment = InStr(Trim(strALineOfText), COMMENT_MARKER) <> 1

        'Note: code to work out dynatables is now in PopulateElement()

        If blnIsNotComment Then
            ' It's a pipe (|) delimited field (we expect)
            arrElementData = Split (strALineOfText, FILE_DELIMITER)
                    
            ' Don't presume all arrays start @ 0!
            lBd = LBound(arrElementData)
                    
            strElementType = ""
            strElementName = ""
            strElementValue = ""
                    
            ' In case we try to access an item in the array that does not exist.
            On Error Resume Next
                
            strElementType = arrElementData(lBd)
            strElementName = arrElementData(lBd+1)
            strElementValue = arrElementData(lBd+2)
                    
            PopulateElement strElementType, strElementName, strElementValue
        End If

    Loop

    Set objWSH = Nothing

End Function

Sub PopulateElement(ByVal strElementType, ByVal strElementName, ByVal strValue)

    Dim objEl               'As DHTML Element
    Dim arrElComponents     'As String
    Dim strTableName        'As String
    Dim strPostFix          'As String
    Dim lngRow              'As Long
    Dim i                   'As Integer

    Select Case UCase(strElementType)
    Case "TEXT", "TEXTAREA" 

         'If the element doesn't exist, test to see if it's a dynatable
         If document.all(strElementName) Is Nothing Then
            arrElComponents = Split(strElementName, "_")
            strTableName = arrElComponents(LBound(arrElComponents))

            If Not document.all(strTableName) Is Nothing Then
                If UCase(document.all(strTableName).TagName) = "TABLE" Then

                    'Get the row number implied by the last section of the element name after _
                    'eg,  "administeredMedication_drugName_3"
                    lngRow = 0
                    strPostFix = arrElComponents(UBound(arrElComponents))
                    If IsNumeric(strPostFix) And InStr(strPostFix, ".") = 0 Then
                        lngRow = CLng(strPostFix)
                    End If
                        
                    'Add rows until the table length = 1 + the postFix (one extra for the header)
                    Do While document.all(strTableName).Rows.Length <= lngRow  
                        Call AddRow(strTableName)
                    Loop
                End If
            End If
         End If

         If Not document.all(strElementName) Is Nothing Then
             document.all(strElementName).value = UnEscapeCharacters(strValue)
         End If
                          
    Case "CHECKBOX"
         If Not document.all(strElementName) Is Nothing Then
            'For some reason, this sometimes doesn't work, so directly modify the HTML
            ' * document.all(strElementName).checked = CBool(strValue)
            If CBool(strValue) Then
                document.all(strElementName).outerHTML = Replace(document.all(strElementName).outerHTML, ">", " checked=true>")
            End If
         End If
             
    Case "SELECT-ONE", "SELECT-MULTIPLE"
         If Not document.all(strElementName) Is Nothing Then
            For i = 0 To document.all(strElementName).options.length - 1
                document.all(strElementName).options(i).selected = CBool(document.all(strElementName).options(i).value = strValue)
            Next
         End If

    Case "RADIO"
         If Not document.all(strElementName) Is Nothing Then
            i = 0 
            Do While Not document.all(strElementName)(i) Is Nothing
                
                'For some reason, this doesn't work, so directly modify the HTML
                ' * document.all(strElementName)(i).checked = CBool(document.all(strElementName)(i).value = strValue)
                If document.all(strElementName)(i).value = strValue Then
                    document.all(strElementName)(i).outerHTML = Replace(document.all(strElementName)(i).outerHTML, ">", " checked=true>")
                End If
                i = i + 1
            Loop
         End If
        
    Case Else
        'Unsupported type
    End Select
    
End Sub

Function StreamElement (ByRef objStream, byRef objEl)
'
' Description:  Write objEls information to the stream, which in our case will be a file.
' Parameters:   
'
    
    Dim blnStreamed     'As Boolean
    Dim strType         'As String
        
    'Trap errors trying to get form item type
    On Error Resume Next
        strType = UCase(CStr(objEl.Type))
        If Err.Number <> 0 Then
            strType = "OTHER"
        End If
    On Error Goto 0

    blnStreamed = True

    Select Case strType
        Case "TEXT", "TEXTAREA"
            If Len(Trim(CStr(objEl.Value))) > 0 Then
                objStream.Writeline strType & FILE_DELIMITER & objEl.Name & FILE_DELIMITER & EscapeCharacters(objEl.Value)
            End If
        Case "CHECKBOX"
            objStream.Writeline strType & FILE_DELIMITER & objEl.Name & FILE_DELIMITER & objEl.Checked
        Case "SELECT-ONE"
            objStream.Writeline strType & FILE_DELIMITER & objEl.Name & FILE_DELIMITER & EscapeCharacters(objEl.Value)
        Case "RADIO"
            If objEl.checked Then
                objStream.Writeline strType & FILE_DELIMITER & objEl.Name & FILE_DELIMITER & EscapeCharacters(objEl.Value)
            End If
        Case Else
            blnStreamed = False     'we don't want to output these details
    End Select             
        
    StreamElement = blnStreamed
        
End Function       
    
Function EscapeCharacters(ByVal strIn) 
'
' Description:  Translate non-printable characters and delimiters into escaped versions.
' Parameters:   
'
    'strIn = Replace(strIn, FILE_DELIMITER, FILE_DELIMITER_SUBSTITUTE)     
    strIn = Replace(strIn, vbCrLf, "\n")                ' So must carriage returns. 
    EscapeCharacters = strIn

End Function
    
' Does the opposite of the above
Function UnEscapeCharacters(strIn)
'
' Description:  Translate escaped characters and delimiters into non-printable versions.
' Parameters:   
'    
    Dim strOut  'As String
        
    'strOut = Replace(strIn, FILE_DELIMITER_SUBSTITUTE, FILE_DELIMITER)
    strOut = Replace(strIn, "\n", vbCrLf)
    UnEscapeCharacters = strOut
    
End Function

Sub FormCancelWithConfirm(strTitle)
'
' Description:  User is now asked to confirm if they want to lose all edits made.
' Parameters:   strTitle - This is the context of closing window (ie: "GO BACK", or "SURE YOU WANT TO QUIT" etc?)    
'
    'Does the opening document have a back button? If so, call it, because if you don't you may find
    'that when window.close executes an hourglass will appear that never goes away!
    If Not opener.document.all("back") Is Nothing Then
        opener.document.all("back").click()
    End If

    If MsgBox("This will close the window." & vbNewLine & _
                "All edits made will be lost." & vbNewLine & VbNewLine & _
                "Are you sure?", _
                vbYesNo, _
                strTitle) = VbYes Then
        window.Close
    End If
End Sub

Sub FormPreview(strPreviewLetterURL)
'
' Description:  Submits the protocol form.
' Parameters:   
'
    Dim objWin      'As Window
    Dim i
    Dim strXMLPath

    'Transfer the form over to a new window
    Set objWin = window.open("about:blank", "RefLetter", "scrollbars, resizable, height=600, width=800, top=20, left=20")        
    While objWin.document.readyState <> "complete"
    Wend
    objWin.document.body.innerHTML = "<div style='display: none'>" & document.forms(FORM_ID).outerHTML & "</div>"

    'Set up the form with a previewProviderID, and submit
    If Not document.all("providerID") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""previewProviderID"" id=""previewProviderID"" value=""" & document.all("providerID").value & """ />"
    End If
    
    If Not document.all("protocolFormXslPath") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""protocolFormXslPath"" id=""protocolFormXslPath"" value=""" & document.all("protocolFormXslPath").value & """ />"
    End If

    If Not document.all("msgtypeid") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""msgtypeid"" id=""msgtypeid"" value=""" & document.all("msgtypeid").value & """ />"
    End If

    If Not document.all("targetSchemaString") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""targetSchemaString"" id=""targetSchemaString"" value=""" & document.all("targetSchemaString").value & """ />"
    End If

    If Not document.all("pvtServerProtocolPath") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""pvtServerProtocolPath"" id=""pvtServerProtocolPath"" value=""" & document.all("pvtServerProtocolPath").value & """ />"
    End If
    
    If Not document.all("pvtServerStylesheetPath") Is Nothing Then
        objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""pvtServerStylesheetPath"" id=""pvtServerStylesheetPath"" value=""" & document.all("pvtServerStylesheetPath").value & """ />"
    End If

    objWin.document.forms(FORM_ID).insertAdjacentHTML "beforeEnd", "<input type=""hidden"" name=""previewtype"" id=""previewtype"" value=""1""/>"        
    objWin.document.forms(FORM_ID).action = strPreviewLetterURL 
    objWin.document.forms(FORM_ID).submit()
    Set objWin = Nothing
End Sub

Function CheckIfObjectExists(object)
'
' Description:  Check if an object exists
' Parameters:   
'

    CheckIfObjectExists = false 

    If jsCheckIfExists(object) = "undefined" Then
        CheckIfObjectExists = false 
    Else
        If Not (object Is Nothing) Then
            CheckIfObjectExists = true
        End If
    End If

End Function

Sub InitialiseNewMessage()
'
' Description:  Initialises form fields on a new message.
' Parameters:   None
'
    Dim strPatientKey               'As String
    Dim strProtocolXML              'As String
    Dim objForm                     'As Object

    Set g_objGpassApp = GetGpassInstance()
    
    'Pre-populate miscellaneous form information
    
    Call PopulateMiscInfo(  document.forms("messageInfo").children("SpecialtyName").value, _
                            document.forms("messageInfo").children("ProtocolName").value, _
                            document.forms("messageInfo").children("ConsultantSurname").value, _
                            document.forms("messageInfo").children("ConsultantForename").value, _
                            document.forms("messageInfo").children("ConsultantID").value, _
                            document.forms("messageInfo").children("HospitalID").value, _
                            document.forms("messageInfo").children("HospitalName").value, _
                            document.forms("messageInfo").children("HospitalAddress").value, _
                            Date())           'default date is today                           

    'Get the patient key where provided
    If Not document.all("patientKey") Is Nothing Then
        strPatientKey = CStr(document.all("patientKey").value)
        'Check for NULL
        If InStr(strPatientKey, "NULL") > 0 Then
            strPatientKey = ""
        End If
    End If
    
    'Get the protocol XML where provided
    If Not document.all("protocolXML") Is Nothing Then
        strProtocolXML = CStr(document.all("protocolXML").xml)
    End If

    If Not g_objGpassApp Is Nothing And Len(strPatientKey) > 0 And Len(strProtocolXML) > 0 Then
        If CheckIfObjectExists(document.all("protocolXML")) Then
            If Len(CStr(document.all("protocolXML").xml)) > 0 Then
                'Prepopulate information from GPASS
                setTimeout "startGpassRetrieval()", 0
            End If
        End If          
    ElseIf CBool(document.all("isThirdParty").value) And Len(strProtocolXML) > 0 Then
        'Prepopulation from third party GP system will already have taken place and we
        'still need to ensure that form elements are not enabled
    Else     
        'We can't collect information from the GP system, so all fields must be made writable
        Call enableFormElements()                
    End If
        
    ' If there is no hcp_id question then clear the reftohcpidscheme & reftohcpidtype hidden fields
    ' so that the HcpId complex type element of the ReferredToHcp complex type gets cleared and deleted.
    ' This is all because we don't want to make the hcp_id question mandatory.
    Set objForm = document.Forms("formref")
    If Not objForm Is Nothing Then
        If objForm("hcp_id") Is Nothing Then
            If Not document.all("reftohcpidscheme") Is Nothing Then
                document.all("reftohcpidscheme").value = ""
            End If
            If Not document.all("reftohcpidtype") Is Nothing Then
                document.all("reftohcpidtype").value = ""
            End If
        End If
    End If
    
    'Deal with the Unique Care Pathway Number if present, Only pre-populated when message is a referral message i.e. msgtype = 0
     If Not document.all("unique_care_pathway_number") Is Nothing Then
        If document.all("msgtype").value = 1 Then
            document.all("unique_care_pathway_number").value = DUMMY_PATHWAY_NUMBER
        End If
     End If

    Set g_objGpassApp = Nothing
End Sub

Sub StartGPassRetrieval()

    Dim strPatientKey   'As String
    Dim strProtocolXML  'As String

    Set g_objGpassApp = GetGpassInstance()

    strPatientKey = ""

    'Get the patient key where provided
    If Not document.all("patientKey") Is Nothing Then
        strPatientKey = CStr(document.all("patientKey").value)
        'Check for NULL
        If InStr(strPatientKey, "NULL") > 0 Then
            strPatientKey = ""
        End If
    End If

    If Len(strPatientKey) > 0 Then
    
        'Get the protocol XML where provided
        If Not document.all("protocolXML") Is Nothing Then
            strProtocolXML = CStr(document.all("protocolXML").xml)
        End If
        
        Call InitialisePrePopMsgWindow()
        Call PopulateGpassReferral(strProtocolXML, strPatientKey, document.forms("messageInfo"))        
    End If          

    Call ClosePrePopMsgWindow()

    If (Not document.forms(FORM_ID).elements.item("chi") Is Nothing) Then
        Call correctCHINumber()
    End If

End Sub

Sub InitialisePrePopMsgWindow()

    Dim i

    m_strPrePopWindowName = "RetrievingInformation"
    Set m_objPrePopWin = window.open("about:blank", m_strPrePopWindowName, "scrollbars=no, resizable=no, titlebar=no,height=100, width=400, top=250, left=250")

    While m_objPrePopWin.document.readyState <> "complete"
    Wend

    m_objPrePopWin.document.title = "Retrieving information from GPASS"
    Set m_objPrePopMsg = m_objPrePopWin.document.createElement("div")
    Set m_objClinEventProgMain = m_objPrePopWin.document.createElement("div")
    Set m_objClinEventProgDetail = m_objPrePopWin.document.createElement("div")
    Set m_objClinEventProgNumbers = m_objPrePopWin.document.createElement("div")

    m_objPrePopWin.document.body.appendChild(m_objPrePopMsg)
    m_objPrePopWin.document.body.appendChild(m_objClinEventProgMain)
    m_objPrePopWin.document.body.appendChild(m_objClinEventProgNumbers)
    m_objClinEventProgMain.appendChild(m_objClinEventProgDetail)

    m_objPrePopMsg.innerHTML = "Retrieving information..."
    	    
End Sub

Sub ClosePrePopMsgWindow()

    On Error Resume Next
    If Not m_objPrePopWin Is Nothing Then
        m_objPrePopWin.close()
    End If
    On Error GoTo 0 

End Sub

Sub PopulateMiscInfo(strSpecialtyName, strProtocolName, strConsultantSurname, strConsultantForename, _
					strRefToID, strHospitalID, strHospitalName, strHospitalAddress, strDate)
'
' Description:  Temporary function to populate miscellaneous fields.
'
' Parameters:   strSpecialtyName        - the specialty being referred to
'               strProtocolName         - the protocol used
'               strConsultantForename   - the given name of the consultant being referred to
'               strConsultantSurname    - the consultant's last name
'               strRefToID              - the consultant/hcp's ID (GMC/HCP Code not HCEID)
'               strHospitalName         - the hospital being referred to
'               strDate                 - the message date
'

    Dim objForm         'As Object

    Set objForm = document.forms(FORM_ID)

    If Not objForm("specialty") Is Nothing Then
        objForm("specialty").value = strSpecialtyName
    'Else
        'Call MsgBox("Faulty protocol XML. Form must have field called 'specialty'.", 16) '16 shows the error icon
    End If 
    
    If Not objForm("protocol") Is Nothing Then
        objForm("protocol").value = strProtocolName
    end if 
    
    If not objForm("HCP_surname") is nothing then
        objForm("HCP_surname").value = strConsultantSurname
    end if 
    
    If not objForm("HCP_forename") Is Nothing Then
        objForm("HCP_forename").value = strConsultantForename
    End If 
    
    If Not objForm("HCP_ID") Is Nothing Then
        objForm("HCP_ID").value = strRefToID
    End If

    If Not objForm("Hospital_ID") Is Nothing Then
        objForm("Hospital_ID").value = strHospitalID
    End If

    If Not objForm("Hospital_Name") Is Nothing Then
        objForm("Hospital_Name").value = strHospitalName
    End If 

    If Not objForm("Hospital_Address") Is Nothing Then
        objForm("Hospital_Address").value = strHospitalAddress
    End If 

    If Not objForm("referral_date") Is Nothing Then
        objForm("referral_date").value = UnambiguousDate(strDate)
    End If

End Sub

Sub enableFormElements()
'
' Description:  Performs any last minute initialisation, and can set all message form 
'               elements to be writable.
' Parameters:   None
'

    Dim objElement                  'As Object
    
    'Make read-only fields writable
    If Not document.forms(FORM_ID) Is Nothing Then
        'Loop through each form element
        For Each objElement In document.forms(FORM_ID)
            ' Only <INPUT type="text" or type="password"> or <TEXTAREA> elements have a readOnly property
            If (UCase(objElement.TagName) = "INPUT") Or (UCase(objElement.TagName) = "TEXTAREA") Then
                If (UCase(objElement.type) = "TEXT") Or (UCase(objElement.type) = "TEXTAREA") Or (UCase(objElement.type) = "PASSWORD") Then
                    ' Don't make the consultant questions writable
                    If Not (UCase(objElement.name) = "HCP_ID" OR UCase(objElement.name) = "HCP_SURNAME" OR UCase(objElement.name) = "HCP_FORENAME" OR UCase(objElement.name) = "HCP_FULLNAME") Then
                        If objElement.readOnly = True Then
                            objElement.readOnly = False
                        End If
                    End If
                End If
            End If
        Next
    End If

    Set objElement = Nothing

End Sub

Function correctCHINumber()     

    Dim strCHI

    'Test Track 50141: Chi number can come through without leading 0. Add it here.
    strCHI = document.forms(FORM_ID).elements.item("chi").value
    If Len(strCHI) = 9 Then
        strCHI = "0" & document.forms(FORM_ID).elements.item("chi").value
        document.forms(FORM_ID).elements.item("chi").value = strCHI
    End If
    	
    'If CHI number is blank, it comes through as 0.  Make the field blank if 
    'this happens.
    If strCHI = "0" Then
        strCHI = ""
        document.forms(FORM_ID).elements.item("chi").value = strCHI
    End If

End Function 

Function BuildPostInfo()
    Dim strHTML	'As String

    strHTML = ""
    strHTML = strHTML & "<div class=hiddenDiv id=parkInfo name=parkInfo>"

    ' If there is no hcp_id question then do not send any reftohcpidtype or scheme so that the
    ' hcpid element is not populated.
    If document.formref("hcp_id") Is Nothing Then
        strHTML = strHTML & "<input type=hidden name='reftohcpidscheme'      id='reftohcpidscheme' value=''/>"
        strHTML = strHTML & "<input type=hidden name='reftohcpidtype'      id='reftohcpidtype' value=''/>"
    Else
        strHTML = strHTML & "<input type=hidden name='reftohcpidscheme'      id='reftohcpidscheme' value='GmcCode'/>"
        strHTML = strHTML & "<input type=hidden name='reftohcpidtype'      id='reftohcpidtype' value='Healthcare Professional'/>"
    End If
    strHTML = strHTML & "<input type=hidden name='specialtyref'  id='specialtyref' value='" & "A1" & "'/>"
    Dim specialtyValue
    If Not document.formref("specialty") Is Nothing Then
        specialtyValue = document.formref("specialty").value
    Else
        specialtyValue = ""
    End If
    
    strHTML = strHTML & "<input type=hidden name='referredtospecialty'  id='referredtospecialty' value='" & specialtyValue & "'/>"

    strHTML = strHTML & "</div>"

    BuildPostInfo = strHTML

End Function

Sub MakeFieldsReadOnly()
'
' Description:  Makes specialty, hospital and protocol name fields read-only.
'               elements to be writable if not in Gpass.
' Parameters:   None
'
    If not document.formref("Specialty") is nothing then
        document.formref("Specialty").readOnly = true
    end if

    If not document.formref("Hospital_ID") is nothing then
        document.formref("Hospital_ID").readOnly = true
    end if

    If not document.formref("Hospital_Name") is nothing then
        document.formref("Hospital_Name").readOnly = true
    end if

    If not document.formref("Protocol") is nothing then
        document.formref("Protocol").readOnly = true
    end if

    If Not document.formref("hcp_id") Is Nothing Then
        document.formref("hcp_id").readonly = True
    End If

    If Not document.formref("hcp_forename") Is Nothing Then
        If Len(Trim(document.formref("hcp_forename").value)) > 0 Then
            document.formref("hcp_forename").readonly = True
        End If
    End If

    If Not document.formref("hcp_surname") Is Nothing Then
        If Len(Trim(document.formref("hcp_surname").value)) > 0 Then
            document.formref("hcp_surname").readonly = True
        End If
    End If

    If Not document.formref("hcp_fullname") Is Nothing Then
        If Len(Trim(document.formref("hcp_fullname").value)) > 0 Then
            document.formref("hcp_fullname").readonly = True
        End If
    End If

End Sub

Function setValidationType()

    Dim blnSex
    Dim blnDob
    Dim lngValidationType

    blnSex = (Not document.all("patient_sex_1") Is Nothing)
    blnDob = (Not document.all("patient_dob") Is Nothing)

    If blnSex And blnDob Then
        lngValidationType = CHI_DOB_SEX
    ElseIf blnSex And (Not blnDob) Then
        lngValidationType = CHI_SEX
    ElseIf blnDob And (Not blnSex) Then
        lngValidationType = CHI_DOB
    ElseIf (Not blnDob) And (Not blnSex) Then
        lngValidationType = CHI_FORMAT_ONLY
    End If  

    setValidationType = lngValidationType

End Function

Function initialiseControls()
    Dim objControls, objControl
    Dim strControls, strControl, strXmlDataIsland, strControlField, strControlValue
    Dim arrControls, arrControl
    Dim intCount, intSubCount

    Const MAIN_SEPARATOR = ""
    Const SEPARATOR = ";"

    Set objControls = document.all("controls")

    If Not objControls Is Nothing Then
        'example: referrerregistrar
        strControls = objControls.value
        arrControls = Split(strcontrols, MAIN_SEPARATOR)

        If IsArray(arrControls) Then
            For intCount=LBound(arrControls) to UBound(arrControls)
                strControl = arrControls(intCount)

                Set objControl = document.all(strControl & "controls")

                If Not objControl Is Nothing Then
                    strControlValue = objControl.value

                    arrControl = Split(strControlValue, SEPARATOR)

                    If IsArray(arrControl) Then
                        strXmlDataIsland = arrControl(0)
                        strControlField = arrControl(1)

                        If ((Len(strXmlDataIsland) > 0) And (Len(strControl) > 0)) Then
                            call setFieldValuesFromControl(strControl, strControlField, True)
                        End If
                    End If
                End If
            Next
        End If
    End If
	
End Function

Sub RunSpellCheck(objTextControl)
'
' Description:  Run spell check on a textarea or text box
' Parameters:   objTextControl - control to spell check
'
    ' For PVT we do not implement spell checking therefore just display a message
    MsgBox "The spell checking functionality is only available within SCI Gateway.", vbInformation + vbOKOnly, "SCI Protocol Verification Tool"
End Sub

Sub RunSpellCheckColumn(sTableId, sColumnId)
'
' Description:  Run spell check on a table or dynatable column
' Parameters:   sTableId - id of table to spell check
'               sColumnId - id of column to spell check
'
    ' For PVT we do not implement spell checking therefore just display a message
    MsgBox "The spell checking functionality is only available within SCI Gateway.", vbInformation + vbOKOnly, "SCI Protocol Verification Tool"
End Sub