﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Xml.Xsl;

using SCI.Gateway.Core.BusinessLogic;
using SCI.Gateway.Core.BusinessLogic.Entity;
using SCI.Gateway.InterfaceTypes;
using SCI.Gateway.Utility;
using SCI.Gateway.Web.Utility;

namespace SCI.Gateway.Web
{
    public class MessageCommit : GatewayPage
    {

        #region GUI Objects

        protected SCI.Gateway.Web.Header headerControl;

        protected System.Web.UI.HtmlControls.HtmlInputHidden messageID;
        protected System.Web.UI.HtmlControls.HtmlInputHidden type;
        protected System.Web.UI.HtmlControls.HtmlInputHidden patientkey;
        protected System.Web.UI.HtmlControls.HtmlInputHidden patientName;  
        protected System.Web.UI.HtmlControls.HtmlInputHidden ThirdPartyAppName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden ThirdPartyProgId;
        protected System.Web.UI.HtmlControls.HtmlInputHidden slotKey;
        protected System.Web.UI.HtmlControls.HtmlInputHidden hospitalName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden specialtyName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden consultantName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden protocolName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden apptDate;
        protected System.Web.UI.HtmlControls.HtmlInputHidden apptTime;
        protected System.Web.UI.HtmlControls.HtmlInputHidden bookAppt;
        protected System.Web.UI.HtmlControls.HtmlInputHidden commitMessage;
        protected System.Web.UI.HtmlControls.HtmlInputHidden DocmanLocationCode;
        protected System.Web.UI.HtmlControls.HtmlInputHidden fldSessionKey;
        protected System.Web.UI.HtmlControls.HtmlInputHidden fldGatewayServices;

        protected System.Web.UI.HtmlControls.HtmlGenericControl message_xml;
        
        #endregion

        #region Private Variables

        private int m_messageId;

        private MessageItem m_message;

        #endregion

        #region Public Properties

        public override string PageTitle
        {
            get 
                {
                    return string.Format(ResourceUserSettingsManager.GetUserSettings(ResourceEnum.PageTitleMessageCommit), m_sessionContext.InstanceName, m_message.MessageType.Name);
                }
        }

        #endregion

        #region Private Properties

        private int MessageId
        {
            get {return m_messageId;}
            set {m_messageId = value;}
        }

        #endregion

        #region Protected Events

        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);
            base.SetJavaScriptFiles("");

            fldSessionKey.Value = this.SessionState.sessionKey;
            fldGatewayServices.Value = this.SessionState.gatewayServiceURL;

            string scriptList = "ThirdParty.vbs;xml.vbs;DocmanMethodCalls.vbs;gpassProtocolObject.vbs;";
            scriptList += "gpassEncounter.vbs;gpassreferral.vbs;gpass.vbs";
            base.SetVBScriptFiles("scripts", scriptList, "MessageCommitScripts");

            DirectoryEntity directoryEntity = DirectoryEntity.FindById(this.SessionState.hceId);

            SetHiddenFields(directoryEntity);

            m_message = MessageItem.FindById(MessageId);
            if (m_message.IsSubmitted)
                throw new GatewayException("Message has already been submitted.");

            //Book the appointment
            if (bookAppt.Value.ToLower() == "true")
            {
                m_message.BookAppointment(SessionState.sessionKey, Convert.ToInt32(slotKey.Value));
            }

            bool canFileInDocman = false;
            bool canImportIntoGPSystem = false;
            bool canImportIntoGpass = false;
            ArrayList parametersForGpass = new ArrayList();

            if (commitMessage.Value.ToLower() == "true")
            {
                m_message.ParkAndSubmit(true, SessionState.sessionKey);
                
                //Determine if we are filing message and attachments in Docman
                canFileInDocman = directoryEntity.CanFileInDocman(m_message.MessageType);

                //Detemine if we are importing into GPASS GP system.  Can only import referrals into GPASS GP system.
                // We check the patientKey.Value to ensure the message was created originally in GPASS
                if ((this.SessionState.inGPASS || ThirdPartyAppName.Value == "GPASS" )
                    && m_message.MessageType.AutoImportOnSubmit 
                    && m_message.MessageType.Key == GatewayConstants.ReferralMessageId
                    && patientkey.Value.Length > 0
                    && (m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_Import, m_message.MessageType.Key)))

                {
                    //This will eventually be removed once Gpass implement the generic GP system interface
                    parametersForGpass = GetImportParametersFromXml(
                        m_message.MessageXml,
                        Server.MapPath(
                            string.Format("{0}{1}\\Schema-{2}\\questions.xml",
                            GatewayLocations.FolderMessageTypes,
                            m_message.MessageType.Name,
                            m_message.SchemaVersion.Version)));
                    if (parametersForGpass.Count > 0)
                    {
                        canImportIntoGpass = true;
                    }
                }
                
                //Detemine if we are importing into non-GPASS GP system
                else if ((this.SessionState.isThirdParty || ThirdPartyAppName.Value != "GPASS")
                        && ThirdPartyAppName.Value != ""
                        && m_message.MessageType.AutoImportOnSubmit
                        && (m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_Import, m_message.MessageType.Key)))
                {
                    canImportIntoGPSystem = true;
                }
            }

            if (canImportIntoGpass || canImportIntoGPSystem || canFileInDocman)
            {
                WriteScript(canImportIntoGpass, canImportIntoGPSystem, canFileInDocman, m_message, parametersForGpass);
            }
            else
            {
                StringBuilder script = new StringBuilder();
                script.Append("<script language='javascript'>");


                PostToUrlBuilder postBuilder = new PostToUrlBuilder();
                if (slotKey.Value.Length > 0)
                {
                    //If we have an appointment reference, send to the appointment details page
                    postBuilder.Uri = new Uri(GatewayLocations.AppointmentDetailsUri, UriKind.Relative);
                    postBuilder.FormName = GatewayLocations.GatewayFormName;
                }
                else
                {
                    //Return to worklist
                    postBuilder = this.CreateReturnToWorklistPost();
                }

                script.Append(postBuilder.GeneratePostCommand());

                //Determine if the message is an AdviceResponse as we need to parse the MessageXml
                //in order to obtain the Advice Request message ID.
                SystemVariable sysVariable = SystemVariable.FindBySystemVariable(SystemVariableName.AdviceResponseMessageTypeId);
                MessageType adviceResponseMessageType = MessageType.FindById(Convert.ToInt32(sysVariable.Value));

                if (m_message.MessageType.Key == adviceResponseMessageType.Key)
                {
                    int requestId = FindRequestIdFromMessageXml();

                    if (requestId != 0)
                    {
                        MessageItem requestMessageItem = MessageItem.FindById(requestId);
                        requestMessageItem.SetStatus(MessageStatus.StatusDeleted);
                    }
                    else
                    {
                        throw new GatewayException("Cannot retrieve a request message ID from the response message.");
                    }   
                    
                }

                script.Append("</script>");
                ClientScript.RegisterStartupScript(GetType(), "MessageCommitComplete", script.ToString());
            }
        }

        protected override void OnInit(EventArgs e)
        {
            InitializeComponent();
            base.OnInit(e);
        }

        #endregion

        #region Private Events
        #endregion

        #region Private Methods

        private void SetHiddenFields(DirectoryEntity directoryEntity)
        {

            if (Request.Form["messageID"] != null)
            {
                MessageId = Convert.ToInt32(Request.Form["messageID"].ToString());
                messageID.Value = MessageId.ToString();
            }

            if (Request.Form["patientkey"] != null)
            {
                patientkey.Value = Request.Form["patientkey"].ToString();
            }

            if (Request.Form["patientName"] != null)
            {
                patientName.Value = Request.Form["patientName"].ToString();
            }

            if (Request.Form["ThirdPartyAppName"] != null)
            {
                ThirdPartyAppName.Value = Request.Form["ThirdPartyAppName"].ToString();
            }

            if (Request.Form["ThirdPartyProgId"] != null)
            {
                ThirdPartyProgId.Value = Request.Form["ThirdPartyProgId"].ToString();
            }

            if (Request.Form["slotKey"] != null)
            {
                slotKey.Value = Request.Form["slotKey"].ToString();
            }

            if (Request.Form["hospitalName"] != null)
            {
                hospitalName.Value = Request.Form["hospitalName"].ToString();
            }

            if (Request.Form["specialtyName"] != null)
            {
                specialtyName.Value = Request.Form["specialtyName"].ToString();
            }

            if (Request.Form["consultantName"] != null)
            {
                consultantName.Value = Request.Form["consultantName"].ToString();
            }

            if (Request.Form["protocolName"] != null)
            {
                protocolName.Value = Request.Form["protocolName"].ToString();
            }

            if (Request.Form["apptDate"] != null)
            {
                apptDate.Value = Request.Form["apptDate"].ToString();
            }

            if (Request.Form["apptTime"] != null)
            {
                apptTime.Value = Request.Form["apptTime"].ToString();
            }

            if (Request.Form["bookAppt"] != null)
            {
                bookAppt.Value = Request.Form["bookAppt"].ToString();
            }

            if (Request.Form["commitMessage"] != null)
            {
                commitMessage.Value = Request.Form["commitMessage"].ToString();
            }

            //Used within DocmanMethodCalls.vbs
            DocmanLocationCode.Value = directoryEntity.FindDocmanLocationCode();
        }

        private void WriteScript(bool canImportIntoGpass, bool canImportIntoGPSystem, bool canFileInDocman, MessageItem message, ArrayList parametersForGpass)
        {
            var script = new StringBuilder();

            //Import into GP system before importing into Docman.

            script.Append("<script language=VBScript>\r");
            script.Append("setTimeout \"DoImport()\", 1000\r");
            script.Append("Sub DoImport()\r");
            
            if (canImportIntoGpass)
            {
                script.Append("Call UpdateGpass(\"" + parametersForGpass[0] + "\", ");
                script.Append("\"" + parametersForGpass[1] + "\", ");
                script.Append("\"" + parametersForGpass[2] + "\", ");
                script.Append("\"" + parametersForGpass[3] + "\", ");
                script.Append("\"" + parametersForGpass[4] + "\", ");
                script.Append("\"" + parametersForGpass[5] + "\", ");
                script.Append("\"" + parametersForGpass[6] + "\", ");
                script.Append("\"" + parametersForGpass[7] + "\", ");
                script.Append("\"" + parametersForGpass[8] + "\", ");
                script.Append("\"" + parametersForGpass[9] + "\", ");
                script.Append("\"" + patientkey.Value + "\")\r");
            }

            if (canImportIntoGPSystem)
            {
                //Set hidden xml field
                message_xml.InnerHtml = message.MessageXml;

                script.Append("Call ImportThirdParty(\"" + message.MessageType.Key + "\", ");
                script.Append("\"" + patientkey.Value + "\", ");
                script.Append("\"" + GatewayLibrary.EncodeForScript(message.MessageLetterHtml) + "\", ");
                script.Append("\"" + ThirdPartyProgId.Value + "\", ");
                script.Append("\"" + ThirdPartyAppName.Value + "\", ");
                script.Append("\"" + message.PatientForename + "\", ");
                script.Append("\"" + message.PatientSurname + "\", ");
                script.Append("\"" + message.PatientChi + "\", ");
                script.Append("\"" + message.PatientDob.GetValueOrDefault().ToShortDateString() + "\", ");
                script.Append("\"" + message.PatientSex + "\", ");
                script.Append("\"" + message.MessageType.Name + "\", ");
                script.Append("True, False)\r");
            }

            if (canFileInDocman)
            {
                string attachmentDetailXml;

                //Get all the attachments for this message
                IList<MessageAttachment> messageAttachmentCollection = MessageAttachment.FindAttachmentsForMessage(message.Key);
                attachmentDetailXml = MessageAttachment.BuildDetailAsString(messageAttachmentCollection,
                                                                                            MessageAttachmentBase.MessageAttachmentsDownloadPathType.SaveToDocman);
                script.Append("Call FileInDocman(\"" + message.PatientSurname + "\", ");
                script.Append("\"" + message.PatientForename + "\", ");
                script.Append("\"" + message.PatientChi + "\", ");
                script.Append("\"" + message.PatientDob.GetValueOrDefault().ToShortDateString() + "\", ");
                script.Append("\"" + message.MessageType.Name + "\", ");
                script.Append("\"" + message.DateLastUpdated.ToShortDateString() + "\", ");
                script.Append("\"" + GatewayLibrary.EncodeForScript(message.MessageXml) + "\", ");
                script.Append("\"" + ((int)message.Priority) + "\", ");
                script.Append("\"" + GatewayLibrary.EncodeForScript(message.MessageLetterHtml) + "\", ");
                script.Append("\"" + GatewayLibrary.EncodeForScript(attachmentDetailXml) + "\", ");
                script.Append("\"" + message.MessageType.DocmanFolder + "\", ");
                script.Append("\"" + message.MessageType.DocmanOrganisation + "\", ");
                script.Append("\"" + message.MessageType.DocmanDepartment + "\", ");
                script.Append(message.FiledSender + ", ");
                script.Append(message.FiledRecipient + ", ");
                script.Append(message.Key + ", ");
                script.Append("False, ");
                script.Append("False, ");
                script.Append("False)\r");
            }

            if (slotKey.Value.Length > 0)
            {
                //If we have an appointment reference, send to the appointment details page
                PostToUrlBuilder postBuilder = new PostToUrlBuilder();
                postBuilder.Uri = new Uri(GatewayLocations.AppointmentDetailsUri, UriKind.Relative);
                postBuilder.FormName = GatewayLocations.GatewayFormName;
                postBuilder.ScriptLanguage = PostToUrlBuilder.Language.VBScript;
                script.Append(postBuilder.GeneratePostCommand());
            }
            else
            {
                //Return to worklist
                PostToUrlBuilder postBuilder = this.CreateReturnToWorklistPost();
                postBuilder.ScriptLanguage = PostToUrlBuilder.Language.VBScript;
                script.Append(postBuilder.GeneratePostCommand());
            }
            script.Append("End Sub\r");
            script.Append("</script>\r");

            ClientScript.RegisterStartupScript(GetType(), "MessageCommitComplete", script.ToString());
        }

        /// <summary>
        /// Create a post object to allow returning to the worklist
        /// </summary>
        /// <returns>The worklist post generator object</returns>
        private PostToUrlBuilder CreateReturnToWorklistPost()
        {
            PostToUrlBuilder postBuilder = new PostToUrlBuilder();
            postBuilder.Uri = new Uri(GatewayLocations.WorklistUriWithMessageType(m_message.MessageType), UriKind.Relative);
            postBuilder.FormName = GatewayLocations.GatewayFormName;
            postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.ResetWorklistState, "false");
            postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.MessageType, m_message.MessageType.Key.ToString());
            postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.SelectReceiveTab, "false");
            postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.PageSelected, "1");
            //postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.SearchFromDate,
            //                                DateTime.Now.AddMonths(-1).ToString(WorklistPage.Constants.WorklistDateFormats.DateSearchFormat));
            //postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.PerformSearch, "true");

            return postBuilder;
        }

        private void InitializeComponent()
        {
            this.Load += new EventHandler(this.Page_Load);
        }

        /// <summary>
        /// Returns an arraylist with 1. refgp_name, 2. refgp_code, 3. maincondition_description, 
        /// 4. referral_expectedoutcome, 5. priority, 6. hospital_name, 7. referredto = "nhs", 8. specialty, 
        /// 9. referraltype, 10. attendancetype = "first"
        /// </summary>
        /// <param name="messageXml">Message xml</param>
        /// <param name="questionsXmlPath">Path to questions.xml file</param>
        /// <returns></returns>
        private static ArrayList GetImportParametersFromXml(string message, string questionsXmlPath)
        {         
            ArrayList values = new ArrayList();
            string value = "";

            //importParameters[0], defaultValues[1], hardCodedValues[2]
            string[,] array = new string[,] {{"refgp_name", "Unknown", ""},
                                             {"refgp_code", "", ""},
                                             {"maincondition_description", "", ""},
                                             {"referral_expectedoutcome", "", ""},
                                             {"priority", "", ""},
                                             {"hospital_name", "SCI Hospital", ""},
                                             {"referredto", "", "NHS"},
                                             {"specialty", "", ""},
                                             {"referraltype", "", ""},
                                             {"attendancetype", "", "first"}};

            XmlDocument questionsXml = new XmlDocument();
            questionsXml.Load(questionsXmlPath);

            XmlDocument messageXml = new XmlDocument();
            messageXml.LoadXml(message);

            XmlNamespaceManager nsManagerMessage = new XmlNamespaceManager(messageXml.NameTable);
            nsManagerMessage.AddNamespace("ref", "http://www.show.scot.nhs.uk/isd/Referral");
            nsManagerMessage.AddNamespace("pbr", "http://www.show.scot.nhs.uk/isd/Referral");
            nsManagerMessage.AddNamespace("gen", GatewayConstants.GeneralNS);

            for (int i = 0; i < array.GetLength(0); i++)
            {
                //Build the path to the question
                string xPath = "/questions/question[@id=\"" + array[i,0] + "\"]";

                //Check for hardcoded value.  If it exists use this instead of value in xml
                if (array[i,2].Length > 0)
                {
                    value = array[i,2];
                }
                else
                {
                    //Get the queryxpath node from the questionsXml.  If it is not there, get the xpath node.
                    XmlNode questionsXmlNode = questionsXml.SelectSingleNode(xPath + "/queryxpath");
                    XmlNode messageXmlNode;

                    if (questionsXmlNode != null)
                    {
                        //Use the queryxpath from the questionsXml to extract data from the messageXml
                        if (questionsXmlNode.InnerText.Length > 0)
                        {
                            messageXmlNode = messageXml.SelectSingleNode(questionsXmlNode.InnerText, nsManagerMessage);
                            if (messageXmlNode != null)
                            {
                                value = messageXmlNode.InnerText;
                            }
                        }
                    }
                    else
                    {

                        questionsXmlNode = questionsXml.SelectSingleNode(xPath + "/xpath");
                        if (questionsXmlNode != null)
                        {
                            //Use the xpath from the questionsXml to extract data from the messageXml
                            if (questionsXmlNode.InnerText.Length > 0)
                            {
                                messageXmlNode = messageXml.SelectSingleNode(questionsXmlNode.InnerText, nsManagerMessage);
                                if (messageXmlNode != null)
                                {
                                    value = messageXmlNode.InnerText;
                                }
                            }
                        }
                    }

                    if (value.Length == 0)
                    {
                        //Get the default value
                        value = array[i,1];
                    }
                }

                //Store the value in the array of parameter values
                values.Add(value);
            }	    
            return values;
        }


        private int FindRequestIdFromMessageXml()
        {
            XmlNode requestMessageIdNode;
            XmlDocument messageXml = new XmlDocument();
            int requestMessageId = 0;

            messageXml.LoadXml(this.m_message.MessageXml);
            XmlNamespaceManager nsManager = XmlUtility.CreateNamespaceManagerFromDocumentNamespaces(messageXml);
            requestMessageIdNode = messageXml.SelectSingleNode("//adRes:RequestMessageId", nsManager);

            if (requestMessageIdNode != null)
            {
                requestMessageId = Convert.ToInt32(requestMessageIdNode.InnerText);
            }

            return requestMessageId;
        }

        #endregion

    }
}
