﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using SCI.Gateway.Web.Utility;
using SCI.Gateway.Core.BusinessLogic;
using SCI.Gateway.InterfaceTypes;
using SCI.Gateway.Utility;

namespace SCI.Gateway.Web
{
    public class MessageOptions : GatewayPage
    {

        #region GUI Objects

        protected SCI.Gateway.Web.Header headerControl;
        protected SCI.Gateway.Web.Controls.SciButton BackButton;
        protected SCI.Gateway.Web.Controls.SciButton ProceedButton;
        protected System.Web.UI.HtmlControls.HtmlForm GatewayForm;
        protected System.Web.UI.WebControls.Label PageDescriptionLabel;
        protected System.Web.UI.WebControls.Label VetResponseLabel;
        protected System.Web.UI.WebControls.Label PleaseSelectLabel;
        protected System.Web.UI.WebControls.Label SeparatorLabel3;
        protected System.Web.UI.WebControls.Label SeparatorLabel4;
        protected System.Web.UI.WebControls.Label NoteLabel1;
        protected System.Web.UI.WebControls.Label NoteLabel2;
        protected System.Web.UI.WebControls.RadioButton SendOnlyRadio;
        protected System.Web.UI.WebControls.RadioButton BookOnlyRadio;
        protected System.Web.UI.WebControls.RadioButton BookAndSendRadio;

        protected System.Web.UI.HtmlControls.HtmlInputHidden messageID;
        protected System.Web.UI.HtmlControls.HtmlInputHidden messageName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden type;
        protected System.Web.UI.HtmlControls.HtmlInputHidden messagePriority;
        protected System.Web.UI.HtmlControls.HtmlInputHidden allowAppts;
        protected System.Web.UI.HtmlControls.HtmlInputHidden vetResponse;
        protected System.Web.UI.HtmlControls.HtmlInputHidden bookAppt;
        protected System.Web.UI.HtmlControls.HtmlInputHidden commitMessage;
        protected System.Web.UI.HtmlControls.HtmlInputHidden patientkey;
        protected System.Web.UI.HtmlControls.HtmlInputHidden patientCHI;
        protected System.Web.UI.HtmlControls.HtmlInputHidden patientName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden refGMCCode;           
        protected System.Web.UI.HtmlControls.HtmlInputHidden hospitalName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden specialtyName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden protocolName;          
        protected System.Web.UI.HtmlControls.HtmlInputHidden ThirdPartyAppName;
        protected System.Web.UI.HtmlControls.HtmlInputHidden ThirdPartyProgId;
        
        #endregion

        #region Private Variables

        private string m_messageName;
        private string m_vetResponse;

        private int m_messageType;

        private bool m_allowAppointments;
        private bool m_bookAppointment;
        private bool m_commitMessage;

        #endregion

        #region Public Properties

        public override string PageTitle
        {
            get 
            {
                return string.Format(ResourceUserSettingsManager.GetUserSettings(ResourceEnum.PageTitleMessageOptions), m_sessionContext.InstanceName, MessageName);
            }
        }

        #endregion

        #region Private Properties

        private string MessageName
        {
            get
            {
                return m_messageName;
            }
            set
            {
                m_messageName = value;
            }
        }

        private int MessageType
        {
            get
            {
                return m_messageType;
            }

            set
            {
                m_messageType = value;
            }
        }

        private bool AllowAppointments
        {
            get
            {
                return m_allowAppointments;
            }
            set
            {
                m_allowAppointments = value;
            }
        }

        private bool BookAppointment
        {
            get
            {
                return m_bookAppointment;
            }
            set
            {
                m_bookAppointment = value;
            }
        }

        private bool CommitMessage
        {
            get
            {
                return m_commitMessage;
            }
            set
            {
                m_commitMessage = value;
            }
        }

        private string VetResponse
        {
            get
            {
                return m_vetResponse;
            }
            set
            {
                m_vetResponse = value;
            }
        }

        #endregion

        #region Protected Events

        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);
            base.SetJavaScriptFiles("messageoptions.js");

            if (!IsPostBack)
            {
                SetHiddenFields();
            }

            PostToUrlBuilder postBuilder = new PostToUrlBuilder();
            postBuilder.Uri = new Uri(GatewayLocations.WorklistUriWithMessageType(MessageType), UriKind.Relative);
            postBuilder.FormName = GatewayLocations.GatewayFormName;
            postBuilder.AddFlag(WorklistPage.Constants.WorklistFlagKeys.MessageType, MessageType.ToString());

            BackButton.ScriptOnClick = postBuilder.GeneratePostCommand();

            ProceedButton.ScriptOnClick = "Proceed();";

            PageDescriptionLabel.Text = MessageName + " Options";
            VetResponseLabel.Text = VetResponse;

            if (AllowAppointments)
            {
                PleaseSelectLabel.Text = "Please select how you wish to continue and click the proceed button. ";

                SendOnlyRadio.Text = "Send " + MessageName + " only";
                if (CommitMessage && !BookAppointment)
                    SendOnlyRadio.Checked = true;

                BookOnlyRadio.Text = "Book appointment only, send " + MessageName + " later";
                if (!CommitMessage && BookAppointment)
                    BookOnlyRadio.Checked = true;

                BookAndSendRadio.Text = "Book appointment and send " + MessageName;
                if (CommitMessage && BookAppointment)
                    BookAndSendRadio.Checked = true;

                NoteLabel1.Text = "Note:";
                NoteLabel2.Text = "If you select \"Send " + MessageName + " only\", you will not be able to subsequently make an electronic booking (where available). This will be the responsibility of the hospital.";
            }
            else
            {
                PleaseSelectLabel.Text = "Please click the proceed button to continue. ";
                SeparatorLabel3.Visible = false;
                SeparatorLabel4.Visible = false;
                SendOnlyRadio.Visible = false;
                BookOnlyRadio.Visible = false;
                BookAndSendRadio.Visible = false;
                NoteLabel1.Visible = false;
                NoteLabel2.Visible = false;
            }

            PleaseSelectLabel.Text += "Alternatively, use the back button to return to the worklist, and keep the message parked as \"In Progress\".";
        }

        protected override void OnInit(EventArgs e)
        {
            InitializeComponent();
            base.OnInit(e);
        }

        #endregion

        #region Private Methods

        private void SetHiddenFields()
        {
            if (Request.Form["messageID"] != null)
            {
                messageID.Value = Request.Form["messageID"];
                if (messageID.Value == "")
                {
                    messageID.Value = "0";
                }
                else
                {
                    MessageItem messageItem = MessageItem.FindById(Int32.Parse(messageID.Value));
                    MessageName = messageItem.MessageType.Name;
                    messageName.Value = MessageName;
                    MessageType = messageItem.MessageType.Key;
                    type.Value = MessageType.ToString();
                }
            }

            if (Request.Form["bookAppt"] != null)
            {
                BookAppointment = Convert.ToBoolean(Request.Form["bookAppt"].ToString());
                bookAppt.Value = BookAppointment.ToString();
            }
            else
            {
                BookAppointment = false;
            }

            if (Request.Form["patientName"] != null)
            {
                patientName.Value = Request.Form["patientName"].ToString();
            }

            if (Request.Form["commitMessage"] != null)
            {
                CommitMessage = Convert.ToBoolean(Request.Form["commitMessage"].ToString());
                commitMessage.Value = CommitMessage.ToString();
            }
            else
            {
                CommitMessage = true;
            }

            if (Request.Form["messagePriority"] != null)
            {
                messagePriority.Value = Request.Form["messagePriority"].ToString();
                if (messagePriority.Value == "")
                {
                    messagePriority.Value = "Routine";
                }
            }

            if (Request.Form["allowAppts"] != null)
            {
                AllowAppointments = Convert.ToBoolean(Request.Form["allowAppts"].ToString());
                allowAppts.Value = AllowAppointments.ToString();
            }

            if (Request.Form["vetResponse"] != null)
            {
                VetResponse = Request.Form["vetResponse"].ToString();
                vetResponse.Value = VetResponse;
            }
            
            if (Request.Form["patientkey"] != null)
            {
                patientkey.Value = Request.Form["patientkey"].ToString();
            }

            if (Request.Form["patientCHI"] != null)
            {
                patientCHI.Value = Request.Form["patientCHI"].ToString();
            }

            if (Request.Form["refGpGmcCode"] != null)
            {
                refGMCCode.Value = Request.Form["refGpGmcCode"].ToString();
            }
          
            if (Request.Form["hospitalName"] != null)
            {
                hospitalName.Value = Request.Form["hospitalName"].ToString();
            }

            if (Request.Form["specialtyName"] != null)
            {
                specialtyName.Value = Request.Form["specialtyName"].ToString();
            }

            if (Request.Form["protocolName"] != null)
            {
                protocolName.Value = Request.Form["protocolName"].ToString();
            }
            
            if (Request.Form["ThirdPartyAppName"] != null)
            {
                ThirdPartyAppName.Value = Request.Form["ThirdPartyAppName"].ToString();
            }

            if (Request.Form["ThirdPartyProgId"] != null)
            {
                ThirdPartyProgId.Value = Request.Form["ThirdPartyProgId"].ToString();
            }

        }

        private void InitializeComponent()
        {
            this.Load += new EventHandler(this.Page_Load);
        }

        #endregion
    }
}
