﻿namespace SCI.Gateway.Web.Message
{

    #region Using Directives

    using System;
    using System.Text;
    using System.Web.UI;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;
    using System.Xml;
    using System.Xml.Xsl;
    using SCI.Gateway.Core.BusinessLogic;
    using SCI.Gateway.Core.BusinessLogic.Entity;
    using SCI.Gateway.Core.BusinessLogic.Isd;
    using SCI.Gateway.Core.BusinessLogic.Protocols;
    using SCI.Gateway.Core.BusinessLogic.ReferrerControl;
    using SCI.Gateway.Core.BusinessLogic.User;
    using SCI.Gateway.InterfaceTypes;
    using SCI.Gateway.OutboundInterface;
    using SCI.Gateway.OutboundInterface.OutpatientWebService;
    using SCI.Gateway.Utility;
    using SCI.Gateway.Web.Controls;
    using SCI.Gateway.Web.Utility;
    using SCI.Gateway.Web.WorklistPage.Constants;
    using System.Collections.Generic;

    #endregion 

    /// <summary>
    /// Edit/Create a new message
    /// </summary>
    public partial class ViewMessage : GatewayPage
    {
        #region Fields

        // constants
        public const string CssStylesheetLowResPath =   "protocolformLowRes.css";
        public const string CssStylesheetPath =         "protocolform.css";
        public const string ProtocolFormTransformPath = "~/stylesheets/protocolform.xslt";

        // static
        private static readonly string ClientScriptUris =
            CreateJavaScriptObject.FromPublicStaticMembers(
                typeof(GatewayLocations),
                new string[]
                        {
                            "LetterUri",
                            "MessageCommitUri",
                            "MessageOptionsUri",
                            "MessageReplyUri",
                            "ViewMessageUri",
                            "WorklistUri",
                            "GatewayFormName"
                        });

        // private memebers
        private	string	        m_consultantHceId;			
        private	bool	        m_makeSupplementary;			
        private	bool	        m_makeTertiary;			
        private	MessageItem	    m_message;			
        private	MessageType	    m_messageType;			
        private	Schema	        m_newMessageSchema;			
        private	MessageItem	    m_originalMessage;			
        private	string	        m_patientDetailXml =        string.Empty;			
        private	string	        m_prePopulationSourceXml;			
        private	ProtocolVersion m_protocolVersion;			
        private	DirectoryEntity	m_provider;			
        private	string	        m_spellingReferenceIds	=	String.Empty;	
        private	TempMessage	    m_tempMessage;			
        private	string	        m_transientDataEntry;
        private bool            m_patientMatched;	
        private	OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod	m_uniqueCarePathwayNumberGenerationMethod	
                    =	OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.NotSelected;	


        // hidden fields for client-script/session state


        #endregion

        #region Properties

        public override string PageTitle
        {
            // we don't need to set anything here as this is set dynamically
            get { return ""; }
        }

        #endregion

        #region Methods

        private bool CheckForInvalidPrePopulation()
        {
            bool invalidPrePop = false;

            if (!string.IsNullOrEmpty(Request["blnPrepopulate"].ToString()))
            {
                invalidPrePop = Convert.ToBoolean(Request["blnPrepopulate"].ToString());
            }
            return invalidPrePop;
        }


        private void RetrievePostInputs()
        {
            messageId.Value =           Request["messageId"];
            messageTypeName.Value =     Request["msgTypeName"];
            tempMessageId.Value =       Request["tempMessageId"];
            msgtypeid.Value =           Request["msgtypeid"];
            protocolid.Value =          Request["protocolid"];
            statusText.Value =          Request["status"];
            consultantForename.Value =  Request["consultantForename"];
            consultantSurname.Value =   Request["consultantSurname"];
            PatientMatched.Value =      GetFormValueAsBoolean("PatientMatched",false).ToString();
            m_patientMatched =          GetFormValueAsBoolean("PatientMatched", false);
            m_consultantHceId =         Request["ConsultantID"];
            m_uniqueCarePathwayNumberGenerationMethod = OpenViewMessageScriptBuilder.FindUniqueCarePathwayNumberGenerationMethodFromString(Request["uniqueCarePathwayNumberGeneration"]);

            int originalMessageId = Convert.ToInt32(Request["originalMessageId"]);
            if (originalMessageId > 0)
            {
                m_originalMessage = MessageItem.FindById(originalMessageId);

                if (bool.Parse(Request["makeTertiary"]))
                {
                    m_makeTertiary = true;
                }
                else if (bool.Parse(Request["makeSupplementary"]))
                {
                    m_makeSupplementary = true;
                }

            }

            string apptBooking = Request["apptBooking"];

            if (!string.IsNullOrEmpty(apptBooking) && !apptBooking.Contains("apptBooking disabled"))
            {
                allowappts.Value = Convert.ToString(true);
            }
            else
            {
                allowappts.Value = Convert.ToString(false);
            }


            int destinationHceId = GetFormValueAsInteger("DestinationEntityId", 0);
            providerid.Value =      destinationHceId.ToString();
            patientkey.Value =      Request["patientkey"];

            int screenWidth =   Convert.ToInt32(Request["ScreenWidth"]);
            int screenHeight =  Convert.ToInt32(Request["ScreenHeight"]);
            if ((screenWidth <= 800) && (screenHeight <= 600))
            {
                messageCssStylesheet.Value =    CssStylesheetLowResPath;
                isLowRes.Value =                true.ToString();
                resetHeight.Value =             "400px";
                originalHeight.Value =          "550px";
            }
            else
            {
                messageCssStylesheet.Value =    CssStylesheetPath;
                isLowRes.Value =                false.ToString();
                resetHeight.Value =             "550px";
                originalHeight.Value =          "720px";
            }

            fldGatewayMessageFolder.Value = ResolveUrl("~/message/parkmessage.htm");

            //Retireve the patient details that may be split into more than one input field
            //and stick them back together!
            //Find out how many
            int fieldCount = 0;

           
                foreach (string key in Request.Form.AllKeys)
                {
                    if (key.Contains("TPIpatientDetailXML_"))
                    {
                        fieldCount++;
                    }
                }
            

            if (fieldCount != 0)
            {
                for (int i = 1; i <= fieldCount; i++)
                {
                    m_patientDetailXml += Request["TPIpatientDetailXML_" + i];
                }
              
                string decodedXml = System.Web.HttpUtility.HtmlDecode(m_patientDetailXml);
                m_patientDetailXml = System.Web.HttpUtility.UrlDecode(decodedXml);
                if (m_patientDetailXml.Length > 0 && Convert.ToBoolean(blnPrepopulate.Value) == false)
                {
                    if (ApplicationConfig.GetConfigSetting(GatewayConfigSettings.LocalisedApplicationName) == "WCCG")
                    {
                        m_patientDetailXml = SetPrePopulationSchemaVersion(m_patientDetailXml);
                    }

                    List<GpXmlSchemaValidationError> errorList = ValidateThirdPartyPatientXml(m_patientDetailXml);
                    bool invalidXml = errorList.Count > 0 ? true : false;
                    if (invalidXml)
                    {
                        invalid.Value = invalidXml.ToString();
                        prePopXml.Value = Server.HtmlDecode(m_patientDetailXml);
                        m_patientDetailXml = string.Empty;
                    }
                }
            }
        }

        private string SetPrePopulationSchemaVersion(string patientDetailsXml)
        {
            string schemaVersionPatienDetails;
            string schemaVersionStylesheetLocation;

            XmlDocument schemaVersionPatienDetailsXml = XmlUtility.CreateDocumentFromXml(patientDetailsXml);
            schemaVersionStylesheetLocation = Server.MapPath("/" + ApplicationConfig.GetConfigSetting(GatewayConfigSettings.ApplicationUrlName) + "/config/PreProcessFullPatientDetails.xslt");
            schemaVersionPatienDetails = XmlUtility.TransformAndCacheDocumentUsingStylesheetFile(schemaVersionPatienDetailsXml, schemaVersionStylesheetLocation);

            return schemaVersionPatienDetails;
        }

        /// <summary>
        /// Validate the patient details passed in by a third party system
        /// </summary>
        /// <param name="thirdPartyPatientXml">The xml to be validated</param>
        /// <returns>If xml is valid returns the xml, otherwise returns an empty string</returns>
        private List<GpXmlSchemaValidationError> ValidateThirdPartyPatientXml(string thirdPartyPatientXml)
        {
            XmlDocument thirdPartyPatient = new XmlDocument();
            thirdPartyPatient.LoadXml(thirdPartyPatientXml);
            XmlNode rootNode = thirdPartyPatient.DocumentElement;
            string schemaVersion = rootNode.Attributes.GetNamedItem("SchemaVersion").Value;
            List<GpXmlSchemaValidationError> errors = new List<GpXmlSchemaValidationError>();

            string schemaUrl = 
                Request.Url.GetComponents(UriComponents.SchemeAndServer, UriFormat.UriEscaped)
                + Page.ResolveUrl(
                    "~/"
                    + ApplicationConfig.GetConfigSetting(GatewayConfigSettings.ThirdPartySchemaUrl)
                    + "-"
                    + schemaVersion
                    + ".xsd");

            try
            {
                SchemaValidation schemaValidation = new SchemaValidation(SessionContext);
                errors = schemaValidation.ValidateAgainstSchema(
                    thirdPartyPatientXml,
                    "http://www.show.scot.nhs.uk/isd/ThirdPartyInterface",
                    schemaUrl, 
                    SessionContext.InstanceName);

                if (errors.Count > 0)
                {
                    foreach (GpXmlSchemaValidationError error in errors)
                    {
                        SessionContext.EventLogger.LogEvent(EventLoggerSeverity.Error, error.ErrorMessage);
                    }
                }
            }
            catch (XmlException exception)
            {
                //Allow the user to carry on
                SessionContext.EventLogger.LogEvent(EventLoggerSeverity.Error, exception);
            }
            return errors;
        }

        private void RetrieveBusinessObjects()
        {
            if (!string.IsNullOrEmpty(messageId.Value) && (Convert.ToInt32(messageId.Value) > 0))
            {
                m_message =             MessageItem.FindById(Convert.ToInt32(messageId.Value));
                m_messageType =         m_message.MessageType;
                m_protocolVersion =     m_message.ProtocolVersion;
                m_provider =            m_message.DestinationEntity;
                m_newMessageSchema =    m_message.SchemaVersion;
            }
            else
            {
                if (!string.IsNullOrEmpty(tempMessageId.Value) && (Convert.ToInt32(tempMessageId.Value) > 0))
                {
                    m_tempMessage = TempMessage.FindById(Convert.ToInt32(tempMessageId.Value));
                }

                m_messageType =         MessageType.FindById(Convert.ToInt32(msgtypeid.Value));
                m_protocolVersion =     ProtocolVersion.FindById(Convert.ToInt32(protocolid.Value));
                m_provider =            DirectoryEntity.FindById(Convert.ToInt32(providerid.Value));
                m_newMessageSchema =    m_provider.FindNewMessageSchema(m_protocolVersion);
            }

            if ((m_message == null) && (m_tempMessage == null))
            {
                m_tempMessage = TempMessage.CreateNew();
            }

        }

        private static void AddHiddenField(XmlNode parent, string fieldId, string fieldValue)
        {
            XmlElement field = parent.OwnerDocument.CreateElement("hdnfld");
            field.SetAttribute("id", fieldId);
            field.SetAttribute("value", fieldValue);
            parent.AppendChild(field);
        }

        private void AddFieldsToXml(XmlDocument protocolXml, XmlDocument patientData)
        {
            XmlElement protocolObjects;

            XmlElement message = protocolXml.CreateElement("messagexml");
            protocolXml.DocumentElement.AppendChild(message);

            if (patientData.DocumentElement != null)
            {
                message.AppendChild(protocolXml.ImportNode(patientData.DocumentElement, true));
                protocolObjects = (XmlElement) protocolXml.SelectSingleNode("//prtobjcts");
            }
            else
            {
                protocolObjects = protocolXml.CreateElement("prtobjcts");
                message.AppendChild(protocolObjects);
            }

            XmlElement hiddenFields = protocolXml.CreateElement("hdnflds");
            protocolObjects.AppendChild(hiddenFields);

            string hospitalName = "";
            string hospitalAddress = "";
            string hospitalCode = "";
            string idScheme = "ProviderID";

            DirectoryEntity nearestLocationOrPractice = m_provider.FindNearestLocationOrGpPractice();

            if (nearestLocationOrPractice != null)
            {
                if (nearestLocationOrPractice.Type.Key == (int)DirectoryEntityType.Location)
                {
                    Location location = Location.GetById(nearestLocationOrPractice.NationalCode);
                    if (location != null)
                    {
                        hospitalCode = location.LocationCode;
                        hospitalName = location.Name;
                        hospitalAddress = location.TrimmedAddressWithPostcode("\n");
                    }
                }
                else if(nearestLocationOrPractice.Type.Key == (int)DirectoryEntityType.GPPractice)
                {
                    GpPractice practice = GpPractice.GetById(nearestLocationOrPractice.NationalCode);
                    if (practice != null)
                    {
                        hospitalCode = practice.PracticeCode;
                        hospitalName = practice.TrimmedAddressLine1;
                        hospitalAddress = practice.TrimmedAddressWithPostcode("\n");
                        idScheme = "Practice Code";
                    }
                }
                if (hospitalName == "")
                {
                    hospitalName = nearestLocationOrPractice.Name;
                }
            }

            string applicationName = string.Empty;
            string applicationId = string.Empty;

            if (SessionState.isThirdParty)
            {
                patientkey.Value =  SessionState.thirdPartyPatientKey;
                applicationName =   SessionState.thirdpartyApplicationName;
                applicationId =     SessionState.thirdPartyProgId;
            }
            else if (patientkey.Value.Length > 0)
            {
                applicationName = "GPASS";
                applicationId = "NewGpass.Application";
            }


            string specialtyRef = "";
            string specialtyName = "";

            if (m_provider.Specialty != null)
            {
                specialtyRef = m_provider.Specialty.NationalCode;
                specialtyName = m_provider.Specialty.Name;
            }

            if (specialtyRef.Trim() == "")
            {
                specialtyRef = "n.k.";
            }
            if (specialtyName.Trim() == "")
            {
                specialtyName = "Specialty name not known"; //this string is used in viewmessage.vbs
            }

            AddHiddenField(hiddenFields, "msgtypeid",           m_messageType.Key.ToString());
            AddHiddenField(hiddenFields, "providerid",          m_provider.Key.ToString());
            AddHiddenField(hiddenFields, "specialtyref",        specialtyRef);
            AddHiddenField(hiddenFields, "protocolid",          m_protocolVersion.Key.ToString());
            AddHiddenField(hiddenFields, "allowappts",          allowappts.Value);
            AddHiddenField(hiddenFields, "hospitalname",        hospitalName);
            AddHiddenField(hiddenFields, "hospitaladdress",     hospitalAddress);
            AddHiddenField(hiddenFields, "hospitalid",          hospitalCode);
            AddHiddenField(hiddenFields, "specialtyname",       specialtyName);
            AddHiddenField(hiddenFields, "referredtospecialty", specialtyName);
            AddHiddenField(hiddenFields, "protocolname",        m_protocolVersion.Protocol.Name);
            AddHiddenField(hiddenFields, "idscheme",            idScheme);

            string gpForename =                 "";
            string gpSurname =                  "";
            string gpId =                       "";
            string consultantForenameValue =    "";
            string consultantSurnameValue =     "";
            string consultantId =               "";
            string refToHcpIdScheme =           "";
            string refToHcpIdType =             "";
            string referToId =                  "";
            string gmcCode =                    m_provider.FindGmcCode();
            string hcpCode =                    m_provider.FindHcpCode();

            if (gmcCode != string.Empty)
            {
                referToId =         gmcCode;
                refToHcpIdScheme =  "GmcCode";
                refToHcpIdType =    "Healthcare Professional";
            }
            else if (hcpCode != string.Empty)
            {
                referToId =         hcpCode;
                refToHcpIdScheme =  "HcpCode";
                refToHcpIdType =    "Healthcare Professional";
            }

            if (m_provider.IsGp)
            {
                gpForename =        consultantForename.Value;
                gpSurname =         consultantSurname.Value;
                gpId =              referToId;
            }
            else
            {
                consultantForenameValue =   consultantForename.Value;
                consultantSurnameValue =    consultantSurname.Value;
                consultantId =              referToId;
            }

            AddHiddenField(hiddenFields, "gpforename",          gpForename);
            AddHiddenField(hiddenFields, "gpsurname",           gpSurname);
            AddHiddenField(hiddenFields, "gpid",                gpId);
            AddHiddenField(hiddenFields, "consultantForename",  consultantForenameValue);
            AddHiddenField(hiddenFields, "consultantSurname",   consultantSurnameValue);
            AddHiddenField(hiddenFields, "consultantid",        consultantId);
            AddHiddenField(hiddenFields, "reftohcpidscheme",    refToHcpIdScheme);
            AddHiddenField(hiddenFields, "reftohcpidtype",      refToHcpIdType);

            //set the date to now for just now as only dealing with new messages so far
            AddHiddenField(hiddenFields, "date",                DateTime.Now.Date.ToShortDateString());

            AddHiddenField(hiddenFields, "isThirdParty",        m_sessionContext.ThirdParty.ToString());
            AddHiddenField(hiddenFields, "ThirdPartyProgId",    applicationId);
            AddHiddenField(hiddenFields, "ThirdPartyAppName",   applicationName);
            AddHiddenField(hiddenFields, "patientkey",          patientkey.Value);

            // Set the document creation date/time. Note that these are only added on a new message
            // and are the time that the form was opened not first parked and not submitted.
            AddHiddenField(hiddenFields, "docdata_create_date", DateTime.Now.ToString("yyyy-MM-dd"));
            AddHiddenField(hiddenFields, "docdata_create_time", DateTime.Now.ToString("HH:mm:ss"));
        }

        private XmlNode PopulateControl(XmlDocument protocolXml,
                                     string controlName)
        {
            XmlNode control = protocolXml.SelectSingleNode("//control[@name='" + controlName + "']");
            if (control != null)
            {
                XmlDocument controlData =
                    Referrer.RetrieveControlData(controlName,
                                                 GatewayUser.FindById(SessionContext.LogonID).UserEntity.Key,
                                                 Server.MapPath("~/config/fields.xml"));
                if (controlData.DocumentElement != null)
                {
                    control.AppendChild(protocolXml.ImportNode(controlData.DocumentElement, true));
                }
            }
            return control;
        }

        private static bool ProtocolContainsControls(XmlNode protocolXml,
                                                     string controlPrefix)
        {
            string upperCaseCharacters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            string lowerCaseCharacters = "abcdefghijklmnopqrstuvwxyz";

            string xpath = "//question[starts-with(translate(@id,'" + upperCaseCharacters + "','" + lowerCaseCharacters +
                           "'),'" + controlPrefix + "_')]";
            XmlNodeList controlNodes = protocolXml.SelectNodes(xpath);
            return (controlNodes.Count > 0);
        }

        private static bool ProtocolContainsSenderDetails(XmlNode protocolXml)
        {
            string xpath = "//SenderDetails";
            XmlNodeList nodeList = protocolXml.SelectNodes(xpath);
            return (nodeList.Count > 0);
        }

        private void PopulateProtocolHtml()
        {
            const string CONTROL_TYPE_REFERRER = "referrer";
            const string CONTROL_TYPE_REGISTRAR = "registrar";

            XmlDocument prePopulationData = new XmlDocument();
            string stylesheetPath = Server.MapPath(ProtocolFormTransformPath);
            XmlDocument protocolXml = m_protocolVersion.CreateDocumentWithValidationInformation(
                Server.MapPath(GatewayLocations.FolderMessageTypes), m_newMessageSchema);

            XmlNode referrerControl = PopulateControl(protocolXml, CONTROL_TYPE_REFERRER);
            XmlNode regGpControl = PopulateControl(protocolXml, CONTROL_TYPE_REGISTRAR);

            string controlTypes = "";
            
            if (referrerControl != null)
            {
                controlTypes = controlTypes + CONTROL_TYPE_REFERRER;
            }

            if (regGpControl != null)
            {
                controlTypes = controlTypes + "" + CONTROL_TYPE_REGISTRAR;
            }

            controls.Value = controlTypes;

            //only want to prepopulate on the first load of the page for new messages with a prepopulation source
            //if (!IsPostBack && (SessionState.isThirdParty || m_originalMessage != null) && m_message == null && m_prePopulationSourceXml != "")
            if ((!IsPostBack
                && m_message == null && 
                m_prePopulationSourceXml != null && m_prePopulationSourceXml.Length > 0) || 
                (IsPostBack&& m_message == null && 
                m_prePopulationSourceXml != null && 
                m_prePopulationSourceXml.Length > 0 && Convert.ToBoolean(blnPrepopulate.Value)))
            {
                PatientPrePopulationParameters patientPrePopulationParameters = new PatientPrePopulationParameters();
                patientPrePopulationParameters.ApplicationName = SessionState.thirdpartyApplicationName;
                string prePopulationDataEntryXml;
                if (m_originalMessage != null)
                {
                    patientPrePopulationParameters.OriginalMessageType = m_originalMessage.MessageType;
                }
                else
                {
                    patientPrePopulationParameters.OriginalMessageType = MessageType.FindById(GatewayConstants.ReferralMessageId);
                }

                if (m_makeSupplementary)
                {
                    patientPrePopulationParameters.PatientPrePopulationMode = PatientPrePopulationMode.Supplementary;
                }
                else
                {
                    patientPrePopulationParameters.PatientPrePopulationMode = PatientPrePopulationMode.GatewayPrePopulationSource;
                }

                patientPrePopulationParameters.PrePopulationSourceXml = m_prePopulationSourceXml;
                patientPrePopulationParameters.NewMessageType = m_messageType;
                patientPrePopulationParameters.NewMessageProtocolXml = protocolXml.InnerXml;
                patientPrePopulationParameters.NewMessageSchema = m_newMessageSchema;

                Patient prePopulation = new Patient(m_sessionContext);
                prePopulationDataEntryXml = prePopulation.CreatePrePopulatedDataEntryXml(patientPrePopulationParameters);

                if (!string.IsNullOrEmpty(prePopulationDataEntryXml))
                {
                    prePopulationData.LoadXml(prePopulationDataEntryXml);
                }
            }

            bool showGpControls = false;
            if (ProtocolContainsControls(protocolXml, "refgp"))
            {
                if ((CurrentUser.UserEntity.IsGp || !ProtocolContainsControls(protocolXml, "reforg")) && !ProtocolContainsSenderDetails(protocolXml))
                {
                    showGpControls = true;
                }
            }

            LiteralControl xmlContent =
                new LiteralControl("<xml id=\"protocolXML\">" + protocolXml.OuterXml + "</xml>\n\n");
            m_messageContent.Controls.Add(xmlContent);

            if ((m_message != null) || !string.IsNullOrEmpty(m_transientDataEntry))
            {
                XmlDocument dataEntry;
                if (!string.IsNullOrEmpty(m_transientDataEntry))
                {
                    dataEntry = XmlUtility.CreateDocumentFromXml(m_transientDataEntry);
                }
                else
                {
                    dataEntry = XmlUtility.CreateDocumentFromXml(m_message.DataEntryXml);
                }
                XmlElement messageXml = protocolXml.CreateElement("messagexml");
                messageXml.AppendChild(protocolXml.ImportNode(dataEntry.SelectSingleNode("/prtobjcts"), true));
                protocolXml.DocumentElement.AppendChild(messageXml);
            }
            else
                AddFieldsToXml(protocolXml, prePopulationData);

            string prtocolFormKey = System.IO.Path.GetFileName(stylesheetPath) + showGpControls.ToString().ToLower();

            //determine it the stylesheet transform is already cached - the filename and showGpControls are used for the cache key
            XslCompiledTransform protocolFormTransform = XmlUtility.RetrieveCachedStylesheetTransform(prtocolFormKey);
            
            string html;

            if (protocolFormTransform == null)
            {
                XmlDocument transformDocument = XmlUtility.CreateDocumentFromFile(stylesheetPath);

                XmlNamespaceManager namespaceManager =
                    XmlUtility.CreateNamespaceManagerFromDocumentNamespaces(transformDocument);

                XmlElement enableSpellcheckVariable =
                    (XmlElement)
                    transformDocument.SelectSingleNode("/xsl:stylesheet/xsl:variable[@name='enablespellcheck']",
                                                       namespaceManager);
                if (enableSpellcheckVariable != null)
                {
                    enableSpellcheckVariable.SetAttribute("select",
                                                          "'" +
                                                          ApplicationConfig.GetConfigSetting(
                                                              GatewayConfigSettings.EnableSpellcheck).ToLower() + "'");
                }

                XmlElement showGpQuestionsVariable =
                    (XmlElement)
                    transformDocument.SelectSingleNode("/xsl:stylesheet/xsl:variable[@name='showgpquestions']",
                                                       namespaceManager);
                if (showGpQuestionsVariable != null)
                {
                    showGpQuestionsVariable.SetAttribute("select", "'" + showGpControls.ToString().ToLower() + "'");
                }

                html = XmlUtility.TransformDocumentUsingStylesheet(protocolXml, transformDocument, prtocolFormKey);
            }
            else
            {
                html = XmlUtility.TransformUsingXslCompiledTransform(protocolXml, protocolFormTransform);
            }

            // the legacy code had multiple forms on a page.  The .Net code already has a single form
            //  that's used, so use this hack to remove the old forms from the transform.
            //  The transform itself cannot be change since it is shared by PVT (*sigh*).
            html = html
                .Replace("<form name=\"formRef\"", "<span id=\"formRef\"")
                .Replace("<form id=\"protocolInfo\"", "<span id=\"protocolInfo\"")
                .Replace("/form", "/span");

            LiteralControl htmlContent = new LiteralControl(html);
            m_messageContent.Controls.Add(htmlContent);

            SetCssStylesheets(ResolveUrl("~/stylesheets/" + messageCssStylesheet.Value));
        }

        private void PreserveGpassEncounterFields()
        {
            string html = "<span id=\"frmCurrentEncounter\">\n";

            foreach (string key in Request.Form.AllKeys)
            {
                if (key.Contains("GPASS_"))
                {
                    html += "<input id='" + key.Replace("GPASS_", "") + "' name='" + key.Replace("GPASS_", "") +
                            "' type='hidden' value='" + Request[key] + "'>\n";
                }
            }
            html += "</span>\n";

            LiteralControl htmlContent = new LiteralControl(html);
            m_messageContent.Controls.Add(htmlContent);
        }

        /// <summary>
        /// Gets the patient key from the hidden field in the XML
        /// </summary>
        /// <returns>Only returns the patientkey if it is set in the hidden field of the xml </returns>
        private void GetPatientKey()
        {
            if (!string.IsNullOrEmpty(messageId.Value) && (Convert.ToInt32(messageId.Value) > 0))
            {
                string patientKeyXpath = "//hdnflds/hdnfld[@id='patientkey']";
                XmlDocument message = XmlUtility.CreateDocumentFromXml(m_message.DataEntryXml);
                XmlNamespaceManager namespaceManager = XmlUtility.CreateNamespaceManagerFromDocumentNamespaces(message);
                XmlNode patientKey = message.SelectSingleNode(patientKeyXpath, namespaceManager);
                if (patientKey.Attributes.GetNamedItem("value").Value.ToLower() != "")
                {
                    patientkey.Value = patientKey.Attributes.GetNamedItem("value").Value.ToLower();
                }
            }
        }

        private void AddClientSideButtonEvents()
        {
            //Attach button
            string args;
            int messageId = 0;
            int tempMessageId = 0;

            if (m_message != null)
            {
                messageId = m_message.Key;
            }
            else
            {
                tempMessageId = m_tempMessage.Key;
            }

            args = messageId + ", " + tempMessageId + ", '', '" + false + "', '', '', " + m_messageType.Key + ",'true'";

            AttachButton.Attributes.Add("onclick", "openAttManagementScr(" + args + ");return false;");

            //Preview letter button
            string letterPage = "Letter.aspx";

            PreviewLetterButton.Attributes.Add("onclick", "PreviewLetter('" + letterPage + "');return false;");

            SpellingButton.Attributes.Add("onclick",
                                          "onCheckSpellingBtnASP('" + m_spellingReferenceIds + "');return false;");
        }

        private void WriteCallFormOpen(bool displayingErrors)
        {
            string clientScript = "<script language=VBScript>\r";
            clientScript += "Call FormOpen(" + displayingErrors + ")\r";
            clientScript += "</script>\r";

            ClientScript.RegisterStartupScript(GetType(), "ClientScript", clientScript);
        }

        private void WriteCallShowErrors()
        {
            string clientScript = "<script language=JScript>\r";
            clientScript += "window.setTimeout('showClientErrors()', 1);\r";
            clientScript += "</script>\r";

            ClientScript.RegisterStartupScript(GetType(), "ShowErrorsScript", clientScript);
        }


        private MessageItem CreateParkedMessage()
        {
            if (m_tempMessage != null)
            {
                return
                    MessageItem.CreateParkedMessage(m_tempMessage,
                                                    Server.MapPath(GatewayLocations.FolderWebsiteRoot),
                                                    Server.MapPath(GatewayLocations.FolderMessageTypes),
                                                    GatewayUser.FindById(SessionContext.LogonID),
                                                    m_messageType,
                                                    m_provider,
                                                    m_protocolVersion,
                                                    Request.Form);
            }
            else
            {
                m_message.Park(
                            Server.MapPath(GatewayLocations.FolderWebsiteRoot),
                            Server.MapPath(GatewayLocations.FolderMessageTypes),
                            Request.Form);
                return m_message;
            }
        }

        private void ParkButton_Click()
        {
            MessageItem newMessage = CreateParkedMessage();

            m_tempMessage = null;
            m_message = newMessage;

            ClientScript.RegisterStartupScript(GetType(), "ParkMessageScript", "submitAfterPark(false, false);", true);
        }

        private void SendButton_Click()
        {
            try
            {
                MessageItem newMessage = CreateParkedMessage();
                newMessage.PrepareForCommit(
                    Server.MapPath(GatewayLocations.FolderWebsiteRoot),
                    Server.MapPath(GatewayLocations.FolderMessageTypes),
                    Request.Form);

                m_tempMessage = null;
                m_message = newMessage;

                string clientScript = "submitAfterPark(true, "
                                      + newMessage.IsAppointmentBooked.ToString().ToLower()
                                      + ");";

                ClientScript.RegisterStartupScript(GetType(), "ParkMessageScript", clientScript, true);
                vetResponse.Value = "The recipient will accept this message.";
            }
            catch (MessageSchemaValidationException schemaValidationException)
            {
                schemaValidationErrors.InnerHtml = schemaValidationException.SchemaErrors.InnerXml;
                m_transientDataEntry = schemaValidationException.MessageWithDataEntry.DataEntryXml;

                WriteCallFormOpen(true);
                WriteCallShowErrors();
            }
        }

        private void HideUnavailableButtons()
        {
            if (!(Convert.ToBoolean(ApplicationConfig.GetConfigSetting(GatewayConfigSettings.EnableSpellcheck)) &&
                  m_spellingReferenceIds.Length > 0))
            {
                SpellingCell.Visible = false;
            }
            
            if (!m_provider.CanReceiveAttachments(m_messageType) || 
                (m_message != null && m_message.IsSubmitted && !m_provider.CanReceiveAttachmentsWithAndAfterMessage(m_messageType)))
            {
                AttachCell.Visible = false;
            }
        }

        private void AddClientScriptReferences()
        {
            StringBuilder vbScriptFiles = new StringBuilder();
            StringBuilder jScriptFiles = new StringBuilder(); 

            vbScriptFiles.Append("gatewayReferral.vbs;");
            vbScriptFiles.Append("gpass.vbs;");
            vbScriptFiles.Append("gpassProtocolObject.vbs;");
            vbScriptFiles.Append("gpassReferral.vbs;");
            vbScriptFiles.Append("gpassPatient.vbs;");
            vbScriptFiles.Append("gpassClinician.vbs;");
            vbScriptFiles.Append("gpassPremises.vbs;");
            vbScriptFiles.Append("gpassPrescribedDrug.vbs;");
            vbScriptFiles.Append("ProtMsg.vbs;");
            vbScriptFiles.Append("validateChiNumber.vbs;");
            vbScriptFiles.Append("validateHCN.vbs;");
            vbScriptFiles.Append("validateDate.vbs;");
            vbScriptFiles.Append("validation-errors.vbs;");
            vbScriptFiles.Append("viewmessage.vbs;");
            vbScriptFiles.Append("xml.vbs");

            SetVBScriptFiles("scripts", vbScriptFiles.ToString(), "ViewMessageVBScripts");

            jScriptFiles.Append("attachment.js;");
            jScriptFiles.Append("date.js;");
            jScriptFiles.Append("Letter.js;");
            jScriptFiles.Append("MouseEvents.js;");
            jScriptFiles.Append("parkmessage.js;");
            jScriptFiles.Append("SubmitSpelling.js;");
            jScriptFiles.Append("RollingNumber.js;");
            jScriptFiles.Append("WorklistControls.js;");
            jScriptFiles.Append("UpdateTitle.js;");
            jScriptFiles.Append("jquery-1.7.js;");
            jScriptFiles.Append("ViewMessage.js");

            SetJavaScriptFiles(jScriptFiles.ToString(), "ViewMessageJScripts");
        }

        private void StoreSpellCheckQuestionIds()
        {
            if (Convert.ToBoolean(ApplicationConfig.GetConfigSetting(GatewayConfigSettings.EnableSpellcheck)))
            {
                XmlDocument protocol = XmlUtility.CreateDocumentFromXml(m_protocolVersion.Xml);
                string xpath =
                    "//question[(@type='text' and @spellCheck='true') or (@type='textArea' and @spellCheck='true') and (not(@readonly = 'true'))] | //column[(@spellCheck='true')]/..";
                XmlNamespaceManager namespaceManager = XmlUtility.CreateNamespaceManagerFromDocumentNamespaces(protocol);
                XmlNodeList spellcheckQuestions = protocol.SelectNodes(xpath, namespaceManager);
                int idCount = 0;

                if (spellcheckQuestions != null)
                {
                    foreach (XmlNode question in spellcheckQuestions)
                    {
                        string type = question.Attributes.GetNamedItem("type").Value;

                        if (type == "table" || type == "dynaTable")
                        {
                            xpath = "./column[(@spellCheck='true')]";
                            XmlNodeList columns = question.SelectNodes(xpath, namespaceManager);
                            foreach (XmlNode column in columns)
                            {
                                idCount++;
                                if (idCount > 1)
                                {
                                    m_spellingReferenceIds += ",";
                                }
                                m_spellingReferenceIds += question.Attributes.GetNamedItem("id").Value +
                                                          "___" + column.Attributes.GetNamedItem("id").Value;
                            }
                        }
                        else
                        {
                            idCount++;
                            if (idCount > 1)
                            {
                                m_spellingReferenceIds += ",";
                            }
                            m_spellingReferenceIds += question.Attributes.GetNamedItem("id").Value;
                        }
                    }
                }
            }
        }

        protected override void OnInit(EventArgs e)
        {
            Load += Page_Load;

            base.OnInit(e);
        }

        protected override void Page_Load(object sender, EventArgs e)
        {
            IgnoreStandardStylesheets();
            base.Page_Load(sender, e);

            if (!IsPostBack)
            {
                RetrievePostInputs();
                WriteCallFormOpen(false);


                fldSessionKey.Value = SessionState.sessionKey;
                fldGatewayServices.Value = SessionState.gatewayServiceURL;
            }
            else if (CheckForInvalidPrePopulation())
            {
                m_patientDetailXml = Request["prePopXml"].ToString();
                RetrievePostInputs();
                providerid.Value = Request["providerid"].ToString();
                WriteCallFormOpen(false);
                invalid.Value = false.ToString();
            }
           

            RetrieveBusinessObjects();

            //if this is a tertiary message then transfer the message letter html and any attachments to new message
            if (m_makeTertiary || m_makeSupplementary)
            {
                if (m_makeTertiary)
                {
                    PopulateAttachmentsFromOriginalMessage();
                }

                m_prePopulationSourceXml = PopulateDetailsFromOriginalMessage();

                if (m_makeSupplementary)
                {
                    m_prePopulationSourceXml = PrePopulationSourceTransform.AppendOriginalMessageDetails(m_prePopulationSourceXml, m_originalMessage);
                    m_prePopulationSourceXml = PrePopulationSourceTransform.AppendDestinationEntityDetails(m_prePopulationSourceXml, m_provider);
                }
            }
            else if ((SessionState.isThirdParty || m_patientMatched) && m_patientDetailXml != null && m_patientDetailXml.Length > 0)
            {
                // Create a prepopulation source xml from the patient third party patient details xml
                m_prePopulationSourceXml = PrePopulationSourceTransform.BuildPrePopulationSourceForPatientDetailXml(Server.MapPath(GatewayLocations.FolderWebsiteRoot), m_patientDetailXml);
            }
            else if(m_uniqueCarePathwayNumberGenerationMethod != OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.NoNumber
                    && m_uniqueCarePathwayNumberGenerationMethod != OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.NotSelected)
            {
                m_prePopulationSourceXml = PrePopulationSourceTransform.BuildPrePopulationSourceForNewMessage(Server.MapPath(GatewayLocations.FolderMessageTypes), m_messageType.Name, m_newMessageSchema.Version);
            }

            PopulatePrePopulationUniqueCarePathwayNumber();

            SetAppointmentBooking();

            //if appointments are allowed check that there are referral points setup before continuing
            if (Convert.ToBoolean(allowappts.Value))
            {
                if (m_provider.Specialty == null)
                {
                    throw new GatewayException(
                        "You cannot send to SCI Outpatients without a specialty in the destination tree");
                }
                if (!ReferralPointsAvailable())
                {
                    throw new GatewayException("This referral point has not been correctly configured at the SCI Outpatients hospital<br>"
                                               + "Please select a different specialty/consultant and try again");
                }
            }

            if (!string.IsNullOrEmpty(buttonClick.Value))
            {
                // PostToUrl discards the regular asp.net button-click event,
                //  so we manually store it in the buttonClick hidden field
                switch (buttonClick.Value)
                {
                    case "Park":
                        ParkButton_Click();
                        buttonClick.Value = "";
                        break;

                    case "Send":
                        SendButton_Click();
                        buttonClick.Value = "";
                        break;

                    default:
                        throw new GatewayException("unrecognised button '" + buttonClick.Value + "'");
                }
            }

            PopulateProtocolHtml();
            PreserveGpassEncounterFields();
            StoreSpellCheckQuestionIds();
            
            if (patientkey.Value == "")
            {
                GetPatientKey();
            }

            if (m_message == null)
            {
                messageId.Value = "0";
                //messageTypeName.Value = "";
            }
            else
            {
                messageId.Value = m_message.Key.ToString();
                messageTypeName.Value = m_message.MessageType.Name;
            }

            tempMessageId.Value = (m_tempMessage == null) ? "0" : m_tempMessage.Key.ToString();

            //Output some .NET classes to the worklist page HTML as client javascript classes for client side use
            ClientScript.RegisterClientScriptBlock(GetType(), "GatewayLocations", ClientScriptUris, true);
            ClientScript.RegisterClientScriptBlock(GetType(), "WorklistFlagKeys", WorklistConstantsAsJavaScript.WorklistFlagKeys, true);

            SetButtonAvailability();
        }

        /// <summary>
        /// Disables buttons depending on permissions
        /// </summary>
        private void SetButtonAvailability()
        {
            int messageTypeId = m_messageType.Key;
            if(!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_AttachFilesOrDocman, messageTypeId))
            {
                AttachButton.Enabled = false;
            }

            if(!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_CreatePark, messageTypeId))
            {
                ParkButton.Enabled = false;
            }

            if(!m_sessionContext.LoggedOnUserHasPermission(UserPermissionDb.SendTab_Send, messageTypeId))
            {
                SendButton.Enabled = false;
            }
        }

        private string PopulateDetailsFromOriginalMessage()
        {
            string result = PrePopulationSourceTransform.BuildPrePopulationSourceFromOriginalMessage(Server.MapPath(GatewayLocations.FolderMessageTypes), m_originalMessage);
            isPrePopulatedFromOriginalMessage.Value = Convert.ToString(true);
            return result;
        }

        private void PopulateAttachmentsFromOriginalMessage()
        {
            Boolean transferSuccess;

            //transfer attachments from original message
            transferSuccess = TempAttachment.Transfer(m_originalMessage.Key, m_tempMessage.Key);
            if (!transferSuccess)
            {
                throw new GatewayException("There was a problem transferring the original attachments to the new " +
                                           m_messageType.Name + " message");
            }

            //add original message letter as attachment
            string letterHtml = m_originalMessage.MessageLetterHtml;

            letterHtml = "<html>\r<head>\r</head>\r<body>\r<center><h1>*** SENT AS ATTACHMENT ***</h1></center>" +
                         letterHtml + "\r</body>\r</html>";
            string fileName = m_originalMessage.MessageType.Name + m_originalMessage.Key + ".html";

            TempAttachment.CreateAttachmentFromHtml(m_tempMessage, letterHtml, fileName, m_originalMessage, CurrentUser);
        }

        private void PopulatePrePopulationUniqueCarePathwayNumber()
        {
            if (m_prePopulationSourceXml != null && m_prePopulationSourceXml.Length > 0)
            {
                switch (m_uniqueCarePathwayNumberGenerationMethod)
                {
                    case OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.ExistingNumber:
                    case OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.NotSelected:
                        // Do nothing - use the value from the original message, which is already set
                        break;
                    case OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.NewNumber:
                        m_prePopulationSourceXml = PrePopulationSourceTransform.ChangeUniqueCarePathwayNumber(m_prePopulationSourceXml, RollingNumber.NextUniqueCarePathwayNumber());
                        break;
                    case OpenViewMessageScriptBuilder.UniqueCarePathwayNumberGenerationMethod.NoNumber:
                        m_prePopulationSourceXml = PrePopulationSourceTransform.ChangeUniqueCarePathwayNumber(m_prePopulationSourceXml, string.Empty);
                        break;
                    default:
                        throw new GatewayException("Unknown unique care pathway number generation method");
                }
            }
        }

        private bool ReferralPointsAvailable()
        {
            string consultantCode = "";
            string destinationUrl = m_provider.FindOutpatientsUrl(m_messageType.Key, 1).Address;
            OutpatientSender outpatientService = new OutpatientSender(m_sessionContext);
            ReferralPointCol referralPoints;

            if (m_consultantHceId != null)
            {
                DirectoryEntity consultant = DirectoryEntity.FindById(Convert.ToInt32(m_consultantHceId));
                consultantCode = consultant.NationalCode;
            }

            referralPoints = outpatientService.OutpatientsGetReferralPoints(destinationUrl,
                                                                            SessionState.sessionKey,
                                                                            m_protocolVersion.Protocol.Key,
                                                                            m_provider.Specialty.NationalCode,
                                                                            consultantCode);
            if (referralPoints.Collection.Length > 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private void SetAppointmentBooking()
        {
            if (!string.IsNullOrEmpty(messageId.Value) && (Convert.ToInt32(messageId.Value) > 0))
            {
                string appointmentBookingXpath = "//hdnflds/hdnfld[@id='allowappts']";
                XmlDocument message = XmlUtility.CreateDocumentFromXml(m_message.DataEntryXml);
                XmlNamespaceManager namespaceManager = XmlUtility.CreateNamespaceManagerFromDocumentNamespaces(message);
                XmlNode appointment = message.SelectSingleNode(appointmentBookingXpath, namespaceManager);
                if (appointment.Attributes.GetNamedItem("value").Value.ToLower() == "true")
                {
                    allowappts.Value = Convert.ToString(true);
                }
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            AddClientScriptReferences();
            AddClientSideButtonEvents();
            HideUnavailableButtons();

            CancelButton.ScriptOnClick = "window.close();return false;";
            ParkButton.ScriptOnClick = "buttonClick.value='Park';FormPark('ViewMessage.aspx');return false;";
            SendButton.ScriptOnClick = "buttonClick.value='Send';FormSubmit('ViewMessage.aspx');return false;";
        }

        #endregion
    }
}
