<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions" xmlns:sup="http://www.show.scot.nhs.uk/isd/Supplementary" xmlns:gen="http://www.show.scot.nhs.uk/isd/General">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	<!-- This file contains templates used to extract details from xml whose schema is of specific types. -->
	<!-- The file is used by the BuildPrepopulationSource.xslt files for message types that can have supplementary messages created from them -->
	<!-- The output file is used as the prepopulation source xml for protocol prepopulation, so the file follows that format -->
	<!-- It does not extract all details, only those for supplementary messages -->

	<!-- Match templates start -->
	<xsl:template match="gen:HcpId">
		<xsl:if test=".">
			<xsl:element name="HcpId" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:call-template name="sup:ExtractGeneralIdType"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	<xsl:template match="gen:HcpName">
		<xsl:element name="Name" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
			<xsl:call-template name="sup:ExtractGeneralPersonalNameType"/>
		</xsl:element>
	</xsl:template>
	<xsl:template match="gen:EmployingOrganisation">
		<xsl:call-template name="sup:ExtractGeneralOrganisationType"/>
	</xsl:template>
	<xsl:template match="gen:OrganisationId">
		<xsl:if test=".">
			<xsl:element name="OrganisationId" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:call-template name="sup:ExtractGeneralIdType"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	<xsl:template match="gen:OrganisationAddress">
		<xsl:element name="Address" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
			<xsl:call-template name="sup:ExtractGeneralAddressType"/>
		</xsl:element>
	</xsl:template>
	<xsl:template match="gen:OrganisationTelecom">
		<xsl:element name="Telephone" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
			<xsl:call-template name="sup:ExtractGeneralTelecomType"/>
		</xsl:element>
	</xsl:template>
	<xsl:template match="gen:HcpSpecialty">
		<xsl:if test=".">
			<xsl:call-template name="sup:ExtractGeneralSpecialtyType"/>
		</xsl:if>
	</xsl:template>
	<!-- Match templates end-->

	<!-- Named templates Start -->
	<xsl:template name="sup:ExtractGeneralHcpDetailType">
		<xsl:apply-templates select="gen:HcpId"/>
		<xsl:apply-templates select="gen:HcpName"/>
		<xsl:apply-templates select="gen:EmployingOrganisation"/>
		<xsl:apply-templates select="gen:HcpSpecialty"/>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralPersonalNameType">
		<xsl:choose>
			<xsl:when test="gen:StructuredName">
				<xsl:if test="gen:StructuredName/gen:Title"><xsl:value-of select="gen:StructuredName/gen:Title"/><![CDATA[ ]]></xsl:if>
				<xsl:value-of select="gen:StructuredName/gen:GivenName"/>
				<xsl:if test="gen:StructuredName/gen:MiddleName"><![CDATA[ ]]><xsl:value-of select="gen:StructuredName/gen:MiddleName"/></xsl:if>&#160;<xsl:value-of select="gen:StructuredName/gen:FamilyName"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="gen:UnstructuredName"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralIdType">
		<xsl:element name="IdValue" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
			<xsl:value-of select="gen:IdValue"/>
		</xsl:element>
		<xsl:if test="gen:IdScheme">
			<xsl:element name="IdScheme" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:value-of select="gen:IdScheme"/>
			</xsl:element>
		</xsl:if>
		<xsl:if test="gen:IdType">
			<xsl:element name="IdType" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:value-of select="gen:IdType"/>
			</xsl:element>
		</xsl:if>
		<xsl:if test="gen:IdSource">
			<xsl:element name="IdSource" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:value-of select="gen:IdSource"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralOrganisationType">
		<xsl:apply-templates select="gen:OrganisationId"/>
		<xsl:if test="gen:OrganisationName">
			<xsl:element name="OrganisationName" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:value-of select="gen:OrganisationName"/>
			</xsl:element>
		</xsl:if>
		<xsl:apply-templates select="gen:OrganisationAddress"/>
		<xsl:apply-templates select="gen:OrganisationTelecom"/>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralAddressType">
		<xsl:choose>
			<xsl:when test="gen:StructuredAddress">
				<xsl:if test="gen:StructuredAddress/gen:PropertyNumber">
					<xsl:element name="PropertyNumber" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
						<xsl:value-of select="gen:StructuredAddress/gen:PropertyNumber"/>
					</xsl:element>
				</xsl:if>
				<!--There can be 1-4 of these-->
				<xsl:for-each select="gen:StructuredAddress/gen:AddressLine">
					<xsl:element name="AddressLine" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
						<xsl:value-of select="."/>
					</xsl:element>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="AddressLine" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
					<xsl:value-of select="gen:UnstructuredAddress"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="gen:PostCode">
			<xsl:element name="PostCode" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:value-of select="gen:PostCode"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralTelecomType">
		<xsl:choose>
			<xsl:when test="gen:StructuredTelecom">
				<xsl:if test="gen:StructuredTelecom/gen:TelecomCountry">
					<xsl:value-of select="gen:StructuredTelecom/gen:TelecomCountry"/><![CDATA[, ]]>
				</xsl:if>
				<xsl:value-of select="gen:StructuredTelecom/gen:TelecomArea"/><![CDATA[ ]]>
				<xsl:value-of select="gen:StructuredTelecom/gen:TelecomNo"/><![CDATA[ ]]>
				<xsl:if test="gen:StructuredTelecom/gen:TelecomExt">
					<xsl:value-of select="gen:StructuredTelecom/gen:TelecomExt"/>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="gen:UnstructuredTelecom"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralSpecialtyType">
		<xsl:element name="SpecialtyName" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
			<xsl:value-of select="gen:SpecialtyName"/>
		</xsl:element>
		<xsl:if test="gen:SpecialtyCode">			<xsl:element name="SpecialtyCode" namespace="http://www.show.scot.nhs.uk/isd/PrePopulationSource">
				<xsl:value-of select="gen:SpecialtyCode"/>			</xsl:element>
		</xsl:if>
	</xsl:template>
	<xsl:template name="sup:ExtractGeneralHcpAbbreviatedType">
		<xsl:apply-templates select="gen:HcpId"/>
		<xsl:apply-templates select="gen:HcpName"/>
		<xsl:apply-templates select="gen:HcpSpecialty"/>
	</xsl:template>
	<!-- Named templates End -->
</xsl:stylesheet>
